# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

""" Unittest for bug #547 """
""" http://bugs.openbossa.org/show_bug.cgi?id=547 """

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QApplication, QTreeWidget, QTreeWidgetItem


class MyMainWindow(unittest.TestCase):
    app = QApplication(sys.argv)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testCase1(self):
        self._tree = QTreeWidget()
        self._tree.setColumnCount(2)
        self._i1 = None
        self._i11 = None

        self._updateTree()
        self.assertEqual(sys.getrefcount(self._i1), 3)
        self.assertEqual(sys.getrefcount(self._i11), 3)

        self._i11.parent().setExpanded(True)
        self._i11.setExpanded(True)

        self._updateTree()
        self.assertEqual(sys.getrefcount(self._i1), 3)
        self.assertEqual(sys.getrefcount(self._i11), 3)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testCase2(self):
        self._tree = QTreeWidget()
        self._tree.setColumnCount(2)
        self._i1 = None
        self._i11 = None

        self._updateTree()
        self.assertEqual(sys.getrefcount(self._i1), 3)
        self.assertEqual(sys.getrefcount(self._i11), 3)

        self._i11.parent().setExpanded(True)
        self._i11.setExpanded(True)

        self.assertEqual(sys.getrefcount(self._i1), 3)
        self.assertEqual(sys.getrefcount(self._i11), 3)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def _updateTree(self):
        self._tree.clear()
        if self._i1 and self._i11:
            self.assertEqual(sys.getrefcount(self._i1), 2)
            self.assertEqual(sys.getrefcount(self._i11), 2)

        self._i1 = QTreeWidgetItem(self._tree, ['1', ])
        self.assertEqual(sys.getrefcount(self._i1), 3)
        self._i11 = QTreeWidgetItem(self._i1, ['11', ])
        self.assertEqual(sys.getrefcount(self._i11), 3)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QStandardItemModel
from PySide6.QtWidgets import (QWidget, QTreeView, QVBoxLayout,
                               QStyledItemDelegate, QHeaderView)
from PySide6.QtCore import Qt
from helper.usesqapplication import UsesQApplication


class Widget(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self.treeView = QTreeView(self)
        layout = QVBoxLayout()
        layout.addWidget(self.treeView)
        self.setLayout(layout)
        self.treeView.setModel(QStandardItemModel())

        self.treeView.model().setHorizontalHeaderLabels(('3', '1', '5'))


class QWidgetTest(UsesQApplication):

    def testDelegates(self):
        widget = Widget()
        t = widget.treeView

        # When calling setItemDelegateForColumn using a separate variable
        # for the second argument (QAbstractItemDelegate), there was no problem
        # on keeping the reference to this object, since the variable was kept
        # alive (case A)
        # Contrary, when instantiating this argument on the function call
        # Using QStyledItemDelegate inside the call the reference of the
        # object was lost, causing a segfault. (case B)

        # Case A
        d = QStyledItemDelegate()
        # Using QStyledItemDelegate from a variable so we keep the reference alive
        # and we encounter no segfault.
        t.setItemDelegateForColumn(0, d)
        # This raised the Segmentation Fault too, because manually destroying
        # the object caused a missing refrence.
        del d
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

        # Getting the delegates
        a = t.itemDelegateForColumn(0)
        self.assertIsInstance(a, QStyledItemDelegate)

        # Case B
        t.setItemDelegateForColumn(1, QStyledItemDelegate())

        # Getting the delegates
        b = t.itemDelegateForColumn(1)
        self.assertIsInstance(b, QStyledItemDelegate)

        # Test for Rows
        t.setItemDelegateForRow(0, QStyledItemDelegate())
        self.assertIsInstance(t.itemDelegateForRow(0), QStyledItemDelegate)

        # Test for general delegate
        t.setItemDelegate(QStyledItemDelegate())
        self.assertIsInstance(t.itemDelegate(), QStyledItemDelegate)

    def testHeader(self):
        tree = QTreeView()
        tree.setHeader(QHeaderView(Qt.Orientation.Horizontal))
        self.assertIsNotNone(tree.header())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
import math

from PySide6.QtCore import (QEasingCurve, QLineF, QMimeData, QPoint, QPointF,
                            QRandomGenerator, QRectF, QTimeLine, Qt)
from PySide6.QtGui import (QBrush, QColor, QDrag, QImage, QPainter, QPixmap,
                           QPen, QTransform)
from PySide6.QtWidgets import (QApplication, QGraphicsItem,
                               QGraphicsItemAnimation, QGraphicsScene,
                               QGraphicsView)

import dragdroprobot_rc  # noqa: F401


def random(boundary):
    return QRandomGenerator.global_().bounded(boundary)


class ColorItem(QGraphicsItem):
    n = 0

    def __init__(self):
        super().__init__()

        self.color = QColor(random(256), random(256), random(256))

        (r, g, b) = (self.color.red(), self.color.green(), self.color.blue())
        self.setToolTip(
            f"QColor({r}, {g}, {b})\nClick and drag this color onto the robot!")
        self.setCursor(Qt.CursorShape.OpenHandCursor)
        self._start_drag_distance = QApplication.startDragDistance()

    def boundingRect(self):
        return QRectF(-15.5, -15.5, 34, 34)

    def paint(self, painter, option, widget):
        painter.setPen(Qt.PenStyle.NoPen)
        painter.setBrush(Qt.GlobalColor.darkGray)
        painter.drawEllipse(-12, -12, 30, 30)
        painter.setPen(QPen(Qt.GlobalColor.black, 1))
        painter.setBrush(QBrush(self.color))
        painter.drawEllipse(-15, -15, 30, 30)

    def mousePressEvent(self, event):
        if event.button() != Qt.MouseButton.LeftButton:
            event.ignore()
            return

        self.setCursor(Qt.CursorShape.ClosedHandCursor)

    def mouseMoveEvent(self, event):
        start = QPointF(event.buttonDownScreenPos(Qt.MouseButton.LeftButton))
        if QLineF(event.screenPos(), start).length() < self._start_drag_distance:
            return

        drag = QDrag(event.widget())
        mime = QMimeData()
        drag.setMimeData(mime)

        ColorItem.n += 1
        if ColorItem.n > 2 and random(3) == 0:
            image = QImage(':/images/head.png')
            mime.setImageData(image)
            drag.setPixmap(QPixmap.fromImage(image).scaled(30, 40))
            drag.setHotSpot(QPoint(15, 30))
        else:
            mime.setColorData(self.color)
            (r, g, b) = (self.color.red(), self.color.green(), self.color.blue())
            mime.setText(f"#{r:02x}{g:02x}{b:02x}")

            pixmap = QPixmap(34, 34)
            pixmap.fill(Qt.GlobalColor.white)

            with QPainter(pixmap) as painter:
                painter.translate(15, 15)
                painter.setRenderHint(QPainter.RenderHint.Antialiasing)
                self.paint(painter, None, None)

            pixmap.setMask(pixmap.createHeuristicMask())

            drag.setPixmap(pixmap)
            drag.setHotSpot(QPoint(15, 20))

        drag.exec()
        self.setCursor(Qt.CursorShape.OpenHandCursor)

    def mouseReleaseEvent(self, event):
        self.setCursor(Qt.CursorShape.OpenHandCursor)


class RobotPart(QGraphicsItem):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.color = QColor(Qt.GlobalColor.lightGray)
        self.pixmap = None
        self._drag_over = False

        self.setAcceptDrops(True)

    def dragEnterEvent(self, event):
        if (event.mimeData().hasColor()
                or (isinstance(self, RobotHead) and event.mimeData().hasImage())):
            event.setAccepted(True)
            self._drag_over = True
            self.update()
        else:
            event.setAccepted(False)

    def dragLeaveEvent(self, event):
        self._drag_over = False
        self.update()

    def dropEvent(self, event):
        self._drag_over = False
        if event.mimeData().hasColor():
            self.color = QColor(event.mimeData().colorData())
        elif event.mimeData().hasImage():
            self.pixmap = QPixmap(event.mimeData().imageData())

        self.update()


class RobotHead(RobotPart):
    def boundingRect(self):
        return QRectF(-15, -50, 30, 50)

    def paint(self, painter, option, widget=None):
        if not self.pixmap:
            painter.setBrush(self._drag_over and self.color.lighter(130) or self.color)
            painter.drawRoundedRect(-10, -30, 20, 30, 25, 25, Qt.SizeMode.RelativeSize)
            painter.setBrush(Qt.GlobalColor.white)
            painter.drawEllipse(-7, -3 - 20, 7, 7)
            painter.drawEllipse(0, -3 - 20, 7, 7)
            painter.setBrush(Qt.GlobalColor.black)
            painter.drawEllipse(-5, -1 - 20, 2, 2)
            painter.drawEllipse(2, -1 - 20, 2, 2)
            painter.setPen(QPen(Qt.GlobalColor.black, 2))
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.drawArc(-6, -2 - 20, 12, 15, 190 * 16, 160 * 16)
        else:
            painter.scale(.2272, .2824)
            painter.drawPixmap(QPointF(-15 * 4.4, -50 * 3.54), self.pixmap)


class RobotTorso(RobotPart):
    def boundingRect(self):
        return QRectF(-30, -20, 60, 60)

    def paint(self, painter, option, widget=None):
        painter.setBrush(self._drag_over and self.color.lighter(130)
                         or self.color)
        painter.drawRoundedRect(-20, -20, 40, 60, 25, 25, Qt.SizeMode.RelativeSize)
        painter.drawEllipse(-25, -20, 20, 20)
        painter.drawEllipse(5, -20, 20, 20)
        painter.drawEllipse(-20, 22, 20, 20)
        painter.drawEllipse(0, 22, 20, 20)


class RobotLimb(RobotPart):
    def boundingRect(self):
        return QRectF(-5, -5, 40, 10)

    def paint(self, painter, option, widget=None):
        painter.setBrush(self._drag_over and self.color.lighter(130) or self.color)
        painter.drawRoundedRect(self.boundingRect(), 50, 50,
                                Qt.SizeMode.RelativeSize)
        painter.drawEllipse(-5, -5, 10, 10)


class Robot(RobotPart):
    def __init__(self):
        super().__init__()

        self.torsoItem = RobotTorso(self)
        self.headItem = RobotHead(self.torsoItem)
        self.upperLeftArmItem = RobotLimb(self.torsoItem)
        self.lowerLeftArmItem = RobotLimb(self.upperLeftArmItem)
        self._upper_right_arm_item = RobotLimb(self.torsoItem)
        self._lower_right_arm_item = RobotLimb(self._upper_right_arm_item)
        self._upper_right_leg_item = RobotLimb(self.torsoItem)
        self._lower_right_leg_item = RobotLimb(self._upper_right_leg_item)
        self.upperLeftLegItem = RobotLimb(self.torsoItem)
        self.lowerLeftLegItem = RobotLimb(self.upperLeftLegItem)

        self.timeline = QTimeLine()
        settings = [
            #         item                  position    rotation at
            #                                x    y    time 0  /  1
            (self.headItem,                  0,  -18,      20,   -20),  # noqa: E241
            (self.upperLeftArmItem,        -15,  -10,     190,   180),  # noqa: E241
            (self.lowerLeftArmItem,         30,    0,      50,    10),  # noqa: E241
            (self._upper_right_arm_item,    15,  -10,     300,   310),  # noqa: E241
            (self._lower_right_arm_item,    30,    0,       0,   -70),  # noqa: E241
            (self._upper_right_leg_item,    10,   32,      40,   120),  # noqa: E241
            (self._lower_right_leg_item,    30,    0,      10,    50),  # noqa: E241
            (self.upperLeftLegItem,        -10,   32,     150,    80),  # noqa: E241
            (self.lowerLeftLegItem,         30,    0,      70,    10),  # noqa: E241
            (self.torsoItem,                 0,    0,       5,   -20)  # noqa: E241
        ]
        self.animations = []
        for item, pos_x, pos_y, rotation1, rotation2 in settings:
            item.setPos(pos_x, pos_y)
            animation = QGraphicsItemAnimation()
            animation.setItem(item)
            animation.setTimeLine(self.timeline)
            animation.setRotationAt(0, rotation1)
            animation.setRotationAt(1, rotation2)
            self.animations.append(animation)
        self.animations[0].setScaleAt(1, 1.1, 1.1)

        self.timeline.setUpdateInterval(1000 / 25)
        curve = QEasingCurve(QEasingCurve.Type.SineCurve)
        self.timeline.setEasingCurve(curve)
        self.timeline.setLoopCount(0)
        self.timeline.setDuration(2000)
        self.timeline.start()

    def boundingRect(self):
        return QRectF()

    def paint(self, painter, option, widget=None):
        pass


if __name__ == '__main__':
    app = QApplication(sys.argv)

    scene = QGraphicsScene(-200, -200, 400, 400)

    for i in range(10):
        item = ColorItem()
        angle = i * 2.0 * math.pi / 10.0
        item.setPos(math.sin(angle) * 150, math.cos(angle) * 150)
        scene.addItem(item)

    robot = Robot()
    robot.setTransform(QTransform().scale(1.2, 1.2))
    robot.setPos(0, -20)
    scene.addItem(robot)

    view = QGraphicsView(scene)
    view.setRenderHint(QPainter.RenderHint.Antialiasing)
    view.setViewportUpdateMode(QGraphicsView.ViewportUpdateMode.BoundingRectViewportUpdate)
    view.setBackgroundBrush(QColor(230, 200, 167))
    view.setWindowTitle("Drag and Drop Robot")
    view.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
import random

from PySide6.QtWidgets import QApplication, QLabel, QPushButton, QVBoxLayout, QWidget
from PySide6.QtCore import Qt, Slot


class MyWidget(QWidget):
    def __init__(self):
        super().__init__()

        self.hello = ["Hallo Welt", "Hei maailma", "Hola Mundo", "Привет мир"]

        self.button = QPushButton("Click me!")
        self.text = QLabel("Hello World")
        self.text.setAlignment(Qt.AlignCenter)

        self.layout = QVBoxLayout()
        self.layout.addWidget(self.text)
        self.layout.addWidget(self.button)
        self.setLayout(self.layout)

        self.button.clicked.connect(self.magic)

    @Slot()
    def magic(self):
        self.text.setText(random.choice(self.hello))


if __name__ == "__main__":
    app = QApplication(sys.argv)

    widget = MyWidget()
    widget.resize(800, 600)
    widget.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, QStringListModel, Signal, Property, Slot

"""Tests PySide6.QtCore.Property()"""


class TestObject(QObject):

    valueChanged = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self._value = -1
        self.valueChanged.connect(self._changed)
        self.getter_called = 0
        self.setter_called = 0
        self.changed_emitted = 0

    @Slot(int)
    def _changed(self):
        self.changed_emitted += 1

    def getValue(self):
        self.getter_called += 1
        return self._value

    def setValue(self, value):
        self.setter_called += 1
        if (self._value != value):
            self._value = value
            self.valueChanged.emit()

    value = Property(int, fget=getValue, fset=setValue,
                     notify=valueChanged)


class TestDerivedObject(QStringListModel):

    valueChanged = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self._value = -1
        self.valueChanged.connect(self._changed)
        self.getter_called = 0
        self.setter_called = 0
        self.changed_emitted = 0

    @Slot(int)
    def _changed(self):
        self.changed_emitted += 1

    def getValue(self):
        self.getter_called += 1
        return self._value

    def setValue(self, value):
        self.setter_called += 1
        if (self._value != value):
            self._value = value
            self.valueChanged.emit()

    value = Property(int, fget=getValue, fset=setValue,
                     notify=valueChanged)


class SpecialProperties(QObject):
    _value = 1

    def __init__(self):
        super().__init__()
        self._readWriteInt = 2
        self._readWriteDecoratedInt = 3

    def readOnlyInt(self):  # Class variable properties
        return 4

    def readWriteInt(self):
        return self._readWriteInt

    def setReadWriteInt(self, v):
        self._readWriteInt = v

    @Property(int)  # Property decorators
    def readOnlyDecoratedInt(self):
        return 5

    @Property(int)
    def readWriteDecoratedInt(self):
        return self._readWriteDecoratedInt

    @readWriteDecoratedInt.setter
    def readWriteDecoratedInt(self, v):
        self._readWriteDecoratedInt = v

    constantValue = Property(int, lambda self: self._value, constant=True)
    readOnlyInt = Property(int, readOnlyInt)
    readWriteInt = Property(int, readWriteInt, fset=setReadWriteInt)


class PropertyTest(unittest.TestCase):

    def test1Object(self):
        """Basic property test."""
        testObject = TestObject()
        v = testObject.value
        self.assertEqual(v, -1)
        self.assertEqual(testObject.getter_called, 1)
        testObject.value = 42
        v = testObject.value
        self.assertEqual(v, 42)
        self.assertEqual(testObject.changed_emitted, 1)
        self.assertEqual(testObject.setter_called, 1)
        self.assertEqual(testObject.getter_called, 2)

    def test2DerivedObject(self):
        """PYSIDE-1255: Run the same test for a class inheriting QObject."""
        testObject = TestDerivedObject()
        v = testObject.value
        self.assertEqual(v, -1)
        self.assertEqual(testObject.getter_called, 1)
        testObject.value = 42
        v = testObject.value
        self.assertEqual(v, 42)
        self.assertEqual(testObject.changed_emitted, 1)
        self.assertEqual(testObject.setter_called, 1)
        self.assertEqual(testObject.getter_called, 2)

    def testSpecialProperties(self):
        """PYSIDE-924, PYSIDE-3227, constant, read-only."""
        testObject = SpecialProperties()
        mo = testObject.metaObject()

        i = mo.indexOfProperty("constantValue")
        self.assertTrue(i != -1)
        metaProperty = mo.property(i)
        self.assertTrue(metaProperty.isConstant())
        self.assertEqual(testObject.constantValue, 1)

        i = mo.indexOfProperty("readWriteInt")
        self.assertTrue(i != -1)
        metaProperty = mo.property(i)
        self.assertTrue(metaProperty.isWritable())
        self.assertEqual(testObject.readWriteInt, 2)
        testObject.readWriteInt = 42
        self.assertEqual(testObject.readWriteInt, 42)

        i = mo.indexOfProperty("readWriteDecoratedInt")
        self.assertTrue(i != -1)
        metaProperty = mo.property(i)
        self.assertTrue(metaProperty.isWritable())
        self.assertEqual(testObject.readWriteDecoratedInt, 3)
        testObject.readWriteDecoratedInt = 42
        self.assertEqual(testObject.readWriteDecoratedInt, 42)

        i = mo.indexOfProperty("readOnlyInt")
        self.assertTrue(i != -1)
        metaProperty = mo.property(i)
        self.assertFalse(metaProperty.isWritable())
        self.assertEqual(testObject.readOnlyInt, 4)
        with self.assertRaises(AttributeError):
            testObject.readOnlyInt = 42

        i = mo.indexOfProperty("readOnlyDecoratedInt")
        self.assertTrue(i != -1)
        metaProperty = mo.property(i)
        self.assertFalse(metaProperty.isWritable())
        self.assertEqual(testObject.readOnlyDecoratedInt, 5)
        with self.assertRaises(AttributeError):
            testObject.readOnlyDecoratedInt = 42


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

# Qt5: this is gone: from PySide6.QtGui import QMacStyle

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QLabel, QStyleFactory

from helper.usesqapplication import UsesQApplication


class QMacStyleTest(UsesQApplication):
    def setUp(self):
        UsesQApplication.setUp(self)
        self.QMacStyle = type(QStyleFactory.create('Macintosh'))

    def testWidgetStyle(self):
        w = QLabel('Hello')
        self.assertTrue(isinstance(w.style(), self.QMacStyle))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtPdfWidgets import QPdfView
from PySide6.QtWidgets import QComboBox
from PySide6.QtCore import Signal, Slot


class ZoomSelector(QComboBox):

    zoom_mode_changed = Signal(QPdfView.ZoomMode)
    zoom_factor_changed = Signal(float)

    def __init__(self, parent):
        super().__init__(parent)
        self.setEditable(True)

        self.addItem("Fit Width")
        self.addItem("Fit Page")
        self.addItem("12%")
        self.addItem("25%")
        self.addItem("33%")
        self.addItem("50%")
        self.addItem("66%")
        self.addItem("75%")
        self.addItem("100%")
        self.addItem("125%")
        self.addItem("150%")
        self.addItem("200%")
        self.addItem("400%")

        self.currentTextChanged.connect(self.on_current_text_changed)
        self.lineEdit().editingFinished.connect(self._editing_finished)

    @Slot(float)
    def set_zoom_factor(self, zoomFactor):
        percent = int(zoomFactor * 100)
        self.setCurrentText(f"{percent}%")

    @Slot()
    def reset(self):
        self.setCurrentIndex(8)  # 100%

    @Slot(str)
    def on_current_text_changed(self, text):
        if text == "Fit Width":
            self.zoom_mode_changed.emit(QPdfView.ZoomMode.FitToWidth)
        elif text == "Fit Page":
            self.zoom_mode_changed.emit(QPdfView.ZoomMode.FitInView)
        elif text.endswith("%"):
            factor = 1.0
            zoom_level = int(text[:-1])
            factor = zoom_level / 100.0
            self.zoom_mode_changed.emit(QPdfView.ZoomMode.Custom)
            self.zoom_factor_changed.emit(factor)

    @Slot()
    def _editing_finished(self):
        self.on_current_text_changed(self.lineEdit().text())



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

from importlib import reload
import os
import shutil
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()


orig_path = Path(__file__).parent
workdir = Path.cwd()
src = orig_path / 'test_module_template.py'
dst = workdir / 'test_module.py'
shutil.copyfile(src, dst)
sys.path.append(os.fspath(workdir))


class TestModuleReloading(unittest.TestCase):

    def testModuleReloading(self):
        '''Test module reloading with on-the-fly modifications.'''
        import test_module
        for i in range(3):
            oldObject = test_module.obj
            self.assertTrue(oldObject is test_module.obj)
            reload(test_module)
            self.assertFalse(oldObject is test_module.obj)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from __feature__ import snake_case  # noqa

"""
PYSIDE-3250: Tests that snake_case works when used in several files
"""

from PySide6.QtWidgets import QWidget  # noqa: E402


def test():
    print(__name__)
    widget = QWidget()
    return widget.size_hint()



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the multimedia/audiooutput example from Qt v5.x, originating from PyQt"""

import sys
from math import pi, sin
from struct import pack

from PySide6.QtCore import (QByteArray, QIODevice, Qt, QSysInfo, QTimer,
                            qWarning, Slot)
from PySide6.QtMultimedia import (QAudioFormat, QAudioSink, QMediaDevices, QtAudio)
from PySide6.QtWidgets import (QApplication, QComboBox, QHBoxLayout, QLabel,
                               QMainWindow, QPushButton, QSlider,
                               QVBoxLayout, QWidget)


class Generator(QIODevice):

    def __init__(self, format, durationUs, sampleRate, parent):
        super().__init__(parent)

        self.m_pos = 0
        self.m_buffer = QByteArray()

        self.generate_data(format, durationUs, sampleRate)

    def start(self):
        self.open(QIODevice.OpenModeFlag.ReadOnly)

    def stop(self):
        self.m_pos = 0
        self.close()

    def generate_data(self, fmt, durationUs, sampleRate):
        pack_format = ''

        sample_size = fmt.bytesPerSample() * 8
        if sample_size == 8:
            if fmt.sampleFormat() == QAudioFormat.SampleFormat.UInt8:
                scaler = lambda x: ((1.0 + x) / 2 * 255)  # noqa: E731
                pack_format = 'B'
            elif fmt.sampleFormat() == QAudioFormat.SampleFormat.Int16:
                scaler = lambda x: x * 127  # noqa: E731
                pack_format = 'b'
        elif sample_size == 16:
            little_endian = QSysInfo.Endian.ByteOrder == QSysInfo.Endian.LittleEndian
            if fmt.sampleFormat() == QAudioFormat.SampleFormat.UInt8:
                scaler = lambda x: (1.0 + x) / 2 * 65535  # noqa: E731
                pack_format = '<H' if little_endian else '>H'
            elif fmt.sampleFormat() == QAudioFormat.SampleFormat.Int16:
                scaler = lambda x: x * 32767  # noqa: E731
                pack_format = '<h' if little_endian else '>h'

        assert pack_format != ''

        channel_bytes = fmt.bytesPerSample()

        length = (fmt.sampleRate() * fmt.channelCount() * channel_bytes) * durationUs // 100000

        self.m_buffer.clear()
        sample_index = 0
        factor = 2 * pi * sampleRate / fmt.sampleRate()

        while length != 0:
            x = sin((sample_index % fmt.sampleRate()) * factor)
            packed = pack(pack_format, int(scaler(x)))

            for _ in range(fmt.channelCount()):
                self.m_buffer.append(packed)
                length -= channel_bytes

            sample_index += 1

    def readData(self, maxlen):
        data = QByteArray()
        total = 0

        while maxlen > total:
            chunk = min(self.m_buffer.size() - self.m_pos, maxlen - total)
            data.append(self.m_buffer.mid(self.m_pos, chunk))
            self.m_pos = (self.m_pos + chunk) % self.m_buffer.size()
            total += chunk

        return data.data()

    def writeData(self, data):
        return 0

    def bytesAvailable(self):
        return self.m_buffer.size() + super(Generator, self).bytesAvailable()


class AudioTest(QMainWindow):

    PUSH_MODE_LABEL = "Enable push mode"
    PULL_MODE_LABEL = "Enable pull mode"
    SUSPEND_LABEL = "Suspend playback"
    RESUME_LABEL = "Resume playback"

    DURATION_SECONDS = 1
    TONE_SAMPLE_RATE_HZ = 600
    DATA_SAMPLE_RATE_HZ = 44100

    def __init__(self, devices):
        super().__init__()

        self.m_devices = devices
        self.m_device = self.m_devices[0]
        self.m_output = None

        self.initialize_window()
        self.initialize_audio()

    def initialize_window(self):

        central_widget = QWidget()
        layout = QVBoxLayout(central_widget)

        self.m_deviceBox = QComboBox()
        self.m_deviceBox.activated[int].connect(self.device_changed)
        for deviceInfo in self.m_devices:
            self.m_deviceBox.addItem(deviceInfo.description(), deviceInfo)

        layout.addWidget(self.m_deviceBox)

        self.m_modeButton = QPushButton()
        self.m_modeButton.clicked.connect(self.toggle_mode)
        self.m_modeButton.setText(self.PUSH_MODE_LABEL)

        layout.addWidget(self.m_modeButton)

        self.m_suspendResumeButton = QPushButton(clicked=self.toggle_suspend_resume)
        self.m_suspendResumeButton.setText(self.SUSPEND_LABEL)

        layout.addWidget(self.m_suspendResumeButton)

        volume_box = QHBoxLayout()
        volume_label = QLabel("Volume:")
        self.m_volumeSlider = QSlider(Qt.Orientation.Horizontal,
                                      minimum=0, maximum=100, singleStep=10)
        self.m_volumeSlider.valueChanged.connect(self.volume_changed)

        volume_box.addWidget(volume_label)
        volume_box.addWidget(self.m_volumeSlider)

        layout.addLayout(volume_box)

        self.setCentralWidget(central_widget)

    def initialize_audio(self):
        self.m_pullTimer = QTimer(self)
        self.m_pullTimer.timeout.connect(self.pull_timer_expired)
        self.m_pullMode = True

        self.m_format = QAudioFormat()
        self.m_format.setSampleRate(self.DATA_SAMPLE_RATE_HZ)
        self.m_format.setChannelCount(1)
        self.m_format.setSampleFormat(QAudioFormat.SampleFormat.Int16)

        info = self.m_devices[0]
        if not info.isFormatSupported(self.m_format):
            qWarning("Default format not supported - trying to use nearest")
            self.m_format = info.nearestFormat(self.m_format)

        self.m_generator = Generator(self.m_format, self.DURATION_SECONDS * 1000000,
                                     self.TONE_SAMPLE_RATE_HZ, self)

        self.create_audio_output()

    def create_audio_output(self):
        self.m_audioSink = QAudioSink(self.m_device, self.m_format)
        self.m_audioSink.stateChanged.connect(self.handle_state_changed)

        self.m_generator.start()
        self.m_audioSink.start(self.m_generator)
        self.m_volumeSlider.setValue(self.m_audioSink.volume() * 100)

    def closeEvent(self, e):
        self.stop()
        e.accept()

    def stop(self):
        self.m_pullTimer.stop()
        self.m_generator.stop()
        self.m_audioSink.stop()

    @Slot(int)
    def device_changed(self, index):
        self.stop()
        self.m_device = self.m_deviceBox.itemData(index)

        self.create_audio_output()

    @Slot(int)
    def volume_changed(self, value):
        if self.m_audioSink is not None:
            self.m_audioSink.setVolume(value / 100.0)

    @Slot()
    def notified(self):
        bytes_free = self.m_audioSink.bytesFree()
        elapsed = self.m_audioSink.elapsedUSecs()
        processed = self.m_audioSink.processedUSecs()
        qWarning(f"bytesFree = {bytes_free}, "
                 f"elapsedUSecs = {elapsed}, "
                 f"processedUSecs = {processed}")

    @Slot()
    def pull_timer_expired(self):
        if self.m_audioSink is not None and self.m_audioSink.state() != QtAudio.State.StoppedState:
            bytes_free = self.m_audioSink.bytesFree()
            if data := self.m_generator.read(bytes_free):
                self.m_output.write(data)

    @Slot()
    def toggle_mode(self):
        self.m_pullTimer.stop()
        self.m_audioSink.stop()

        if self.m_pullMode:
            self.m_modeButton.setText(self.PULL_MODE_LABEL)
            self.m_output = self.m_audioSink.start()
            self.m_pullMode = False
            self.m_pullTimer.start(20)
        else:
            self.m_modeButton.setText(self.PUSH_MODE_LABEL)
            self.m_pullMode = True
            self.m_audioSink.start(self.m_generator)

        self.m_suspendResumeButton.setText(self.SUSPEND_LABEL)

    @Slot()
    def toggle_suspend_resume(self):
        if self.m_audioSink.state() == QtAudio.State.SuspendedState:
            qWarning("status: Suspended, resume()")
            self.m_audioSink.resume()
            self.m_suspendResumeButton.setText(self.SUSPEND_LABEL)
        elif self.m_audioSink.state() == QtAudio.State.ActiveState:
            qWarning("status: Active, suspend()")
            self.m_audioSink.suspend()
            self.m_suspendResumeButton.setText(self.RESUME_LABEL)
        elif self.m_audioSink.state() == QtAudio.State.StoppedState:
            qWarning("status: Stopped, resume()")
            self.m_audioSink.resume()
            self.m_suspendResumeButton.setText(self.SUSPEND_LABEL)
        elif self.m_audioSink.state() == QtAudio.State.IdleState:
            qWarning("status: IdleState")

    state_map = {
        QtAudio.State.ActiveState: "ActiveState",
        QtAudio.State.SuspendedState: "SuspendedState",
        QtAudio.State.StoppedState: "StoppedState",
        QtAudio.State.IdleState: "IdleState"}

    @Slot("QtAudio::State")
    def handle_state_changed(self, state):
        state = self.state_map.get(state, 'Unknown')
        qWarning(f"state = {state}")


if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.setApplicationName("Audio Output Test")

    devices = QMediaDevices.audioOutputs()
    if not devices:
        print('No audio outputs found.', file=sys.stderr)
        sys.exit(-1)

    audio = AudioTest(devices)
    audio.show()

    sys.exit(app.exec())



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QAbstractListModel, Qt, Slot
from PySide6.QtMultimedia import QWindowCapture


class WindowListModel(QAbstractListModel):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._window_list = QWindowCapture.capturableWindows()

    def rowCount(self, QModelIndex):
        return len(self._window_list)

    def data(self, index, role):
        if role == Qt.ItemDataRole.DisplayRole:
            window = self._window_list[index.row()]
            return window.description()
        return None

    def window(self, index):
        return self._window_list[index.row()]

    @Slot()
    def populate(self):
        self.beginResetModel()
        self._window_list = QWindowCapture.capturableWindows()
        self.endResetModel()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations


import platform
import sys
from pathlib import Path
from email.generator import Generator

from .log import log
from .options import OPTION, CommandMixin
from .utils import is_64bit
from .wheel_utils import get_package_version, get_qt_version, macos_plat_name

wheel_module_exists = False


try:

    from packaging import tags
    from wheel import __version__ as wheel_version
    from wheel.bdist_wheel import bdist_wheel as _bdist_wheel
    from wheel.bdist_wheel import get_abi_tag, get_platform
    from wheel.bdist_wheel import safer_name as _safer_name

    wheel_module_exists = True
except Exception as e:
    _bdist_wheel, wheel_version = type, ""  # dummy to make class statement happy
    log.warning(f"***** Exception while trying to prepare bdist_wheel override class: {e}. "
                "Skipping wheel overriding.")


def get_bdist_wheel_override():
    return PysideBuildWheel if wheel_module_exists else None


class PysideBuildWheel(_bdist_wheel, CommandMixin):

    user_options = (_bdist_wheel.user_options + CommandMixin.mixin_user_options
                    if wheel_module_exists else None)

    def __init__(self, *args, **kwargs):
        self.command_name = "bdist_wheel"
        self._package_version = None
        _bdist_wheel.__init__(self, *args, **kwargs)
        CommandMixin.__init__(self)

    def finalize_options(self):
        CommandMixin.mixin_finalize_options(self)
        if sys.platform == 'darwin' and not self.is_cross_compile:
            # Override the platform name to contain the correct
            # minimum deployment target.
            # This is used in the final wheel name.
            self.plat_name = macos_plat_name()

        # When limited API is requested, notify bdist_wheel to
        # create a properly named package, which will contain
        # the initial cpython version we support.
        limited_api_enabled = OPTION["LIMITED_API"] == 'yes'
        if limited_api_enabled:
            self.py_limited_api = "cp37"

        self._package_version = get_package_version()

        _bdist_wheel.finalize_options(self)

    @property
    def wheel_dist_name(self):
        # Slightly modified version of wheel's wheel_dist_name
        # method, to add the Qt version as well.
        # Example:
        #   PySide6-6.3-6.3.2-cp36-abi3-macosx_10_10_intel.whl
        # The PySide6 version is "6.3".
        # The Qt version built against is "6.3.2".
        wheel_version = f"{self._package_version}-{get_qt_version()}"
        components = (_safer_name(self.distribution.get_name()), wheel_version)
        if self.build_number:
            components += (self.build_number,)
        return '-'.join(components)

    # Modify the returned wheel tag tuple to use correct python version
    # info when cross-compiling. We use the python info extracted from
    # the shiboken python config test.
    # setuptools / wheel don't support cross compiling out of the box
    # at the moment. Relevant discussion at
    # https://discuss.python.org/t/towards-standardizing-cross-compiling/10357
    def get_cross_compiling_tag_tuple(self, tag_tuple):
        (old_impl, old_abi_tag, plat_name) = tag_tuple

        # Compute tag from the python version that the build command
        # queried.
        build_command = self.get_finalized_command('build')
        python_target_info = build_command.python_target_info['python_info']

        impl = 'no-py-ver-impl-available'
        abi = 'no-abi-tag-info-available'
        py_version = python_target_info['version'].split('.')
        py_version_major, py_version_minor, _ = py_version

        so_abi = python_target_info['so_abi']
        if so_abi and so_abi.startswith('cpython-'):
            interpreter_name, cp_version = so_abi.split('-')[:2]
            impl_name = tags.INTERPRETER_SHORT_NAMES.get(interpreter_name) or interpreter_name
            impl_ver = f"{py_version_major}{py_version_minor}"
            impl = impl_name + impl_ver
            abi = f'cp{cp_version}'
        tag_tuple = (impl, abi, plat_name)
        return tag_tuple

    # Adjust wheel tag for limited api and cross compilation.
    @staticmethod
    def adjust_cross_compiled_many_linux_tag(old_tag):
        (old_impl, old_abi_tag, old_plat_name) = old_tag

        new_plat_name = old_plat_name

        # TODO: Detect glibc version instead. We're abusing the
        # manylinux2014 tag here, just like we did with manylinux1
        # for x86_64 builds.
        many_linux_prefix = 'manylinux2014'
        linux_prefix = "linux_"
        if old_plat_name.startswith(linux_prefix):
            # Extract the arch suffix like -armv7l or -aarch64
            _index = old_plat_name.index(linux_prefix) + len(linux_prefix)
            plat_name_arch_suffix = old_plat_name[_index:]

            new_plat_name = f"{many_linux_prefix}_{plat_name_arch_suffix}"

        tag = (old_impl, old_abi_tag, new_plat_name)
        return tag

    # Adjust wheel tag for limited api and cross compilation.
    def adjust_tag_and_supported_tags(self, old_tag, supported_tags):
        tag = old_tag
        (old_impl, old_abi_tag, old_plat_name) = old_tag

        # Get new tag for cross builds.
        if self.is_cross_compile:
            tag = self.get_cross_compiling_tag_tuple(old_tag)

        # Use PEP600 for manylinux wheel name
        # For Qt6 we know RHEL 8.4 is the base linux platform,
        # and has GLIBC 2.28.
        # This will generate a name that contains:
        #     manylinux_2_28
        # TODO: Add actual distro detection, instead of
        # relying on limited_api option.
        if (old_plat_name in ('linux-x86_64', 'linux_x86_64')
                and is_64bit()
                and self.py_limited_api):
            _, _version = platform.libc_ver()
            glibc = _version.replace(".", "_")
            tag = (old_impl, old_abi_tag, f"manylinux_{glibc}_x86_64")

        # Set manylinux tag for cross-compiled builds when targeting
        # limited api.
        if self.is_cross_compile and self.py_limited_api:
            tag = self.adjust_cross_compiled_many_linux_tag(tag)

        # Reset the abi name and python versions supported by this wheel
        # when targeting limited API. This is the same code that's
        # in get_tag(), but done later after our own customizations.
        if self.py_limited_api and old_impl.startswith('cp3'):
            (_, _, adjusted_plat_name) = tag
            impl = self.py_limited_api
            abi_tag = 'abi3'
            tag = (impl, abi_tag, adjusted_plat_name)

        # If building for limited API or we created a new tag, add it
        # to the list of supported tags.
        if tag != old_tag or self.py_limited_api:
            supported_tags.append(tag)
        return tag

    # A slightly modified copy of get_tag from bdist_wheel.py, to allow
    # adjusting the returned tag without triggering an assert. Otherwise
    # we would have to rename wheels manually.
    # Copy is up-to-date since commit
    # 0acd203cd896afec7f715aa2ff5980a403459a3b in the wheel repo.
    def get_tag(self):
        # bdist sets self.plat_name if unset, we should only use it for purepy
        # wheels if the user supplied it.
        if self.plat_name_supplied:
            plat_name = self.plat_name
        elif self.root_is_pure:
            plat_name = 'any'
        else:
            # macosx contains system version in platform name so need special handle
            if self.plat_name and not self.plat_name.startswith("macosx"):
                plat_name = self.plat_name
            else:
                # on macOS always limit the platform name to comply with any
                # c-extension modules in bdist_dir, since the user can specify
                # a higher MACOSX_DEPLOYMENT_TARGET via tools like CMake

                # on other platforms, and on macOS if there are no c-extension
                # modules, use the default platform name.
                plat_name = get_platform(self.bdist_dir)

            if plat_name in ('linux-x86_64', 'linux_x86_64') and not is_64bit():
                plat_name = 'linux_i686'

        plat_name = plat_name.lower().replace('-', '_').replace('.', '_')

        if self.root_is_pure:
            if self.universal:
                impl = 'py3'
            else:
                impl = self.python_tag
            tag = (impl, 'none', plat_name)
        else:
            impl_name = tags.interpreter_name()
            impl_ver = tags.interpreter_version()
            impl = impl_name + impl_ver
            # We don't work on CPython 3.1, 3.0.
            if self.py_limited_api and (impl_name + impl_ver).startswith('cp3'):
                impl = self.py_limited_api
                abi_tag = 'abi3'
            else:
                abi_tag = str(get_abi_tag()).lower()
            tag = (impl, abi_tag, plat_name)
            # issue gh-374: allow overriding plat_name
            supported_tags = [(t.interpreter, t.abi, plat_name)
                              for t in tags.sys_tags()]
            # PySide's custom override.
            tag = self.adjust_tag_and_supported_tags(tag, supported_tags)
            assert tag in supported_tags, (f"would build wheel with unsupported tag {tag}")
        return tag

    # Copy of get_tag from bdist_wheel.py, to write a triplet Tag
    # only once for the limited_api case.
    def write_wheelfile(self, wheelfile_base, generator=f'bdist_wheel ({wheel_version})'):
        from email.message import Message
        msg = Message()
        msg['Wheel-Version'] = '1.0'  # of the spec
        msg['Generator'] = generator
        msg['Root-Is-Purelib'] = str(self.root_is_pure).lower()
        if self.build_number is not None:
            msg['Build'] = self.build_number

        # Doesn't work for bdist_wininst
        impl_tag, abi_tag, plat_tag = self.get_tag()
        # To enable pypi upload we are adjusting the wheel name
        pypi_ready = True if OPTION["LIMITED_API"] else False

        def writeTag(impl):
            for abi in abi_tag.split('.'):
                for plat in plat_tag.split('.'):
                    msg['Tag'] = '-'.join((impl, abi, plat))
        if pypi_ready:
            writeTag(impl_tag)
        else:
            for impl in impl_tag.split('.'):
                writeTag(impl)

        wheelfile_path = Path(wheelfile_base) / 'WHEEL'
        log.info(f'creating {wheelfile_path}')
        with open(wheelfile_path, 'w') as f:
            Generator(f, maxheaderlen=0).flatten(msg)


if not wheel_module_exists:
    del PysideBuildWheel



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test case for QObject::connectNotify()'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal, SIGNAL, SLOT
from helper.usesqapplication import UsesQApplication


def cute_slot():
    pass


class Obj(QObject):

    foo = Signal()

    def __init__(self):
        super().__init__()
        self.con_notified = False
        self.dis_notified = False
        self.signal = ""

    def connectNotify(self, signal):
        self.con_notified = True
        self.signal = signal

    def disconnectNotify(self, signal):
        self.dis_notified = True

    def reset(self):
        self.con_notified = False
        self.dis_notified = False


class TestQObjectConnectNotify(UsesQApplication):
    '''Test case for QObject::connectNotify'''
    def setUp(self):
        UsesQApplication.setUp(self)
        self.called = False

    def tearDown(self):
        UsesQApplication.tearDown(self)

    def testBasic(self):
        sender = Obj()
        receiver = QObject()
        sender.destroyed.connect(receiver.deleteLater)
        self.assertTrue(sender.con_notified)
        self.assertEqual(sender.signal.methodSignature(), "destroyed()")
        self.assertTrue(sender.destroyed.disconnect(receiver.deleteLater))
        self.assertTrue(sender.dis_notified)

    def testBasicString(self):
        sender = Obj()
        receiver = QObject()
        sender.connect(SIGNAL("destroyed()"), receiver, SLOT("deleteLater()"))
        self.assertTrue(sender.con_notified)
        self.assertEqual(sender.signal.methodSignature(), "destroyed(QObject*)")
        self.assertTrue(sender.disconnect(SIGNAL("destroyed()"), receiver, SLOT("deleteLater()")))
        self.assertTrue(sender.dis_notified)

    def testPySignal(self):
        sender = Obj()
        receiver = QObject()
        sender.foo.connect(receiver.deleteLater)
        self.assertTrue(sender.con_notified)
        self.assertTrue(sender.foo.disconnect(receiver.deleteLater))
        self.assertTrue(sender.dis_notified)

    def testPySlots(self):
        sender = Obj()
        sender.destroyed.connect(cute_slot)
        self.assertTrue(sender.con_notified)
        self.assertTrue(sender.destroyed.disconnect(cute_slot))
        self.assertTrue(sender.dis_notified)

    def testpyAll(self):
        sender = Obj()
        sender.foo.connect(cute_slot)
        self.assertTrue(sender.con_notified)
        self.assertTrue(sender.foo.disconnect(cute_slot))
        self.assertTrue(sender.dis_notified)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import importlib
import os
import re
import shutil
import sys
import tempfile
import unittest
import subprocess
from pathlib import Path
from unittest import mock
from unittest.mock import patch

sys.path.append(os.fspath(Path(__file__).resolve().parents[2]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)


class DeployTestBase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pyside_root = Path(__file__).parents[5].resolve()
        cls.example_root = cls.pyside_root / "examples"
        cls.temp_dir = tempfile.mkdtemp()
        cls.current_dir = Path.cwd()
        cls.pyside_wheel = Path("/tmp/PySide6-6.5.0a1-6.5.0-cp37-abi3-android_x86_64.whl")
        cls.shiboken_wheel = Path("/tmp/shiboken6-6.5.0a1-6.5.0-cp37-abi3-android_x86_64.whl")
        cls.ndk_path = Path("/tmp/android_sdk/ndk/25.2.9519653")
        cls.sdk_path = Path("/tmp/android_sdk")
        pyside_tools = cls.pyside_root / "sources" / "pyside-tools"

        # install extra python dependencies
        android_requirements_file = pyside_tools / "requirements-android.txt"
        with open(android_requirements_file, 'r', encoding='UTF-8') as file:
            while line := file.readline():
                dependent_package = line.rstrip().split('==')[0]
                if not bool(importlib.util.find_spec(dependent_package)):
                    command = [sys.executable, "-m", "pip", "install", dependent_package]
                    subprocess.run(command)

        sys.path.append(str(pyside_tools))
        cls.deploy_lib = importlib.import_module("deploy_lib")
        cls.android_deploy = importlib.import_module("android_deploy")
        sys.modules["android_deploy"] = cls.android_deploy

        # required for comparing long strings
        cls.maxDiff = None

        # print no outputs to stdout
        sys.stdout = mock.MagicMock()

    def tearDown(self) -> None:
        super().tearDown()
        os.chdir(self.current_dir)

    @classmethod
    def tearDownClass(cls) -> None:
        shutil.rmtree(Path(cls.temp_dir))


@patch("deploy_lib.android.android_config.extract_and_copy_jar")
class TestPySide6AndroidDeployWidgets(DeployTestBase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        example_widget_application = cls.example_root / "gui" / "analogclock"
        cls.temp_example = Path(
            shutil.copytree(example_widget_application, Path(cls.temp_dir) / "analogclock")
        ).resolve()

    def setUp(self):
        os.chdir(self.temp_example)
        self.config_file = self.temp_example / "pysidedeploy.spec"
        self.buildozer_config = self.temp_example / "buildozer.spec"

    @patch("deploy_lib.android.android_config.AndroidConfig._find_local_libs")
    @patch("deploy_lib.android.android_config.AndroidConfig._find_dependent_qt_modules")
    @patch("deploy_lib.android.android_config.find_qtlibs_in_wheel")
    def test_dry_run(self, mock_qtlibs, mock_extraqtmodules, mock_local_libs, mock_extract_jar):
        mock_qtlibs.return_value = self.pyside_wheel / "PySide6/Qt/lib"
        mock_extraqtmodules.return_value = []
        dependent_plugins = ["platforms_qtforandroid",
                             "platforminputcontexts_qtvirtualkeyboardplugin",
                             "iconengines_qsvgicon"]
        mock_local_libs.return_value = [], dependent_plugins
        self.android_deploy.main(name="android_app", shiboken_wheel=self.shiboken_wheel,
                                 pyside_wheel=self.pyside_wheel, ndk_path=self.ndk_path,
                                 dry_run=True, force=True)

        self.assertEqual(mock_extract_jar.call_count, 0)
        self.assertEqual(mock_qtlibs.call_count, 1)
        self.assertEqual(mock_extraqtmodules.call_count, 1)
        self.assertEqual(mock_local_libs.call_count, 1)

    @patch("deploy_lib.android.buildozer.BuildozerConfig._BuildozerConfig__find_jars")
    @patch("deploy_lib.android.android_config.AndroidConfig.recipes_exist")
    @patch("deploy_lib.android.android_config.AndroidConfig._find_dependent_qt_modules")
    @patch("deploy_lib.android.android_config.find_qtlibs_in_wheel")
    def test_config(self, mock_qtlibs, mock_extraqtmodules, mock_recipes_exist, mock_find_jars,
                    mock_extract_jar):
        jar_dir = "tmp/jar/PySide6/jar"
        mock_extract_jar.return_value = Path(jar_dir)
        mock_qtlibs.return_value = self.pyside_wheel / "PySide6/Qt/lib"
        mock_extraqtmodules.return_value = []
        mock_recipes_exist.return_value = True
        jars, init_classes = ["/tmp/jar/PySide6/jar/Qt6Android.jar",
                              "/tmp/jar/PySide6/jar/Qt6AndroidBindings.jar"], []
        mock_find_jars.return_value = jars, init_classes

        self.android_deploy.main(name="android_app", shiboken_wheel=self.shiboken_wheel,
                                 pyside_wheel=self.pyside_wheel, ndk_path=self.ndk_path,
                                 init=True, force=True, keep_deployment_files=True)

        self.assertEqual(mock_extract_jar.call_count, 1)
        self.assertEqual(mock_qtlibs.call_count, 1)
        self.assertEqual(mock_extraqtmodules.call_count, 1)
        self.assertEqual(mock_recipes_exist.call_count, 1)
        self.assertEqual(mock_find_jars.call_count, 1)
        self.assertTrue(self.config_file.exists())
        self.assertTrue(self.buildozer_config.exists())

        # test config file contents
        config_obj = self.deploy_lib.BaseConfig(config_file=self.config_file)
        self.assertTrue(config_obj.get_value("app", "input_file").endswith("main.py"))
        self.assertEqual(config_obj.get_value("python", "android_packages"),
                         "buildozer==1.5.0,cython==0.29.33")
        self.assertEqual(config_obj.get_value("android", "wheel_pyside"),
                         str(self.pyside_wheel.resolve()))
        self.assertEqual(config_obj.get_value("android", "wheel_shiboken"),
                         str(self.shiboken_wheel.resolve()))
        self.assertEqual(config_obj.get_value("buildozer", "mode"), "debug")
        self.assertEqual(config_obj.get_value("buildozer", "recipe_dir"),
                         '')
        self.assertEqual(config_obj.get_value("buildozer", "jars_dir"),
                         str(self.temp_example / jar_dir))
        self.assertIn(str(self.ndk_path), config_obj.get_value("buildozer", "ndk_path"))
        self.assertEqual(config_obj.get_value("buildozer", "sdk_path"), '')
        expected_modules = {"Core", "Gui"}
        obtained_modules = set(config_obj.get_value("qt", "modules").split(","))
        self.assertEqual(obtained_modules, expected_modules)
        expected_local_libs = ""
        self.assertEqual(config_obj.get_value("buildozer", "local_libs"),
                         expected_local_libs)
        self.assertEqual(config_obj.get_value("buildozer", "arch"), "x86_64")

        # test buildozer config file contents
        buildozer_config_obj = self.deploy_lib.BaseConfig(config_file=self.buildozer_config)
        obtained_jars = set(buildozer_config_obj.get_value("app", "android.add_jars").split(','))
        expected_jars = set(jars)
        self.assertEqual(obtained_jars, expected_jars)
        obtained_extra_args = buildozer_config_obj.get_value("app", "p4a.extra_args")
        extra_args_patrn = re.compile("--qt-libs=(?P<modules>.*) --load-local-libs="
                                      "(?P<local_libs>.*) --init-classes=(?P<init_classes>.*)")
        match = extra_args_patrn.search(obtained_extra_args)
        obtained_modules = match.group("modules").split(',')
        obtained_local_libs = match.group("local_libs")
        obtained_init_classes = match.group("init_classes")
        self.assertEqual(set(obtained_modules), expected_modules)
        self.assertEqual(obtained_local_libs, expected_local_libs)
        self.assertEqual(obtained_init_classes, '')
        expected_include_exts = "py,png,jpg,kv,atlas,qml,js"
        obtained_include_exts = buildozer_config_obj.get_value("app", "source.include_exts")
        self.assertEqual(expected_include_exts, obtained_include_exts)

        self.config_file.unlink()
        self.buildozer_config.unlink()

    def test_errors(self, mock_extract_jar):
        # test when no shiboken wheel is passed
        with self.assertRaises(RuntimeError) as context:
            self.android_deploy.main(name="android_app", pyside_wheel=self.pyside_wheel,
                                     ndk_path=self.ndk_path, force=True)
        self.assertTrue("Unable to find shiboken6 Android wheel" in str(context.exception))

        # test when no PySide wheel is passed
        with self.assertRaises(RuntimeError) as context:
            self.android_deploy.main(name="android_app", shiboken_wheel=self.shiboken_wheel,
                                     ndk_path=self.ndk_path, force=True)
        self.assertTrue("Unable to find PySide6 Android wheel" in str(context.exception))

        # test if wheel name is correct
        pyside_wheel_temp = Path("/tmp/"
                                 "PySide6-6.8.0.1+commercial-6.8.0.1-cp311-cp311-android_fake.whl")
        with self.assertRaises(RuntimeError) as context:
            self.android_deploy.main(name="android_app", shiboken_wheel=self.shiboken_wheel,
                                     pyside_wheel=pyside_wheel_temp, ndk_path=self.ndk_path,
                                     force=True)
        self.assertTrue("PySide wheel corrupted" in str(context.exception))

        # test when cwd() is not project_dir
        os.chdir(self.current_dir)
        with self.assertRaises(RuntimeError) as context:
            self.android_deploy.main(name="android_app", shiboken_wheel=self.shiboken_wheel,
                                     pyside_wheel=self.pyside_wheel, init=True, force=True)
        self.assertTrue("For Android deployment to work" in str(context.exception))

    @patch("deploy_lib.android.buildozer.BuildozerConfig._BuildozerConfig__find_jars")
    @patch("deploy_lib.android.android_config.AndroidConfig.recipes_exist")
    @patch("deploy_lib.android.android_config.AndroidConfig._find_dependent_qt_modules")
    @patch("deploy_lib.android.android_config.find_qtlibs_in_wheel")
    @patch("deploy_lib.android.android_config.download_android_ndk")
    def test_no_ndk_passed(self, mock_ndk, mock_qtlibs, mock_extraqtmodules, mock_recipes_exist,
                           mock_find_jars, mock_extract_jar):
        jar_dir = "tmp/jar/PySide6/jar"
        mock_extract_jar.return_value = Path(jar_dir)
        mock_qtlibs.return_value = self.pyside_wheel / "PySide6/Qt/lib"
        mock_extraqtmodules.return_value = []
        mock_recipes_exist.return_value = True
        jars, init_classes = ["/tmp/jar/PySide6/jar/Qt6Android.jar",
                              "/tmp/jar/PySide6/jar/Qt6AndroidBindings.jar"], []
        mock_find_jars.return_value = jars, init_classes
        mock_ndk.return_value = Path("/tmp/android_ndk")

        self.android_deploy.main(name="android_app", shiboken_wheel=self.shiboken_wheel,
                                 pyside_wheel=self.pyside_wheel,
                                 init=True, force=True,
                                 keep_deployment_files=True)

        # test config file contents
        config_obj = self.deploy_lib.BaseConfig(config_file=self.config_file)
        self.assertEqual(config_obj.get_value("buildozer", "sdk_path"), '')
        self.assertEqual(config_obj.get_value("buildozer", "ndk_path"), "/tmp/android_ndk")


@patch("deploy_lib.config.run_qmlimportscanner")
@patch("deploy_lib.android.android_config.extract_and_copy_jar")
class TestPySide6AndroidDeployQml(DeployTestBase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # setting up example
        example_qml_application = cls.example_root / "quick" / "models" / "stringlistmodel"
        cls.temp_qml_example = Path(
            shutil.copytree(example_qml_application, Path(cls.temp_dir) / "stringlistmodel")
        ).resolve()

    def setUp(self):
        os.chdir(self.temp_qml_example)
        self.config_file = self.temp_qml_example / "pysidedeploy.spec"
        self.buildozer_config_file = self.temp_qml_example / "buildozer.spec"
        (self.temp_qml_example / "stringlistmodel.py").rename(self.temp_qml_example / "main.py")
        (self.temp_qml_example / "stringlistmodel.pyproject").unlink()

    @patch("deploy_lib.android.android_config.AndroidConfig._find_local_libs")
    @patch("deploy_lib.android.buildozer.BuildozerConfig._BuildozerConfig__find_jars")
    @patch("deploy_lib.android.android_config.AndroidConfig.recipes_exist")
    @patch("deploy_lib.android.android_config.AndroidConfig._find_dependent_qt_modules")
    @patch("deploy_lib.android.android_config.find_qtlibs_in_wheel")
    def test_config_with_Qml(self, mock_qtlibs, mock_extraqtmodules, mock_recipes_exist,
                             mock_find_jars, mock_local_libs, mock_extract_jar,
                             mock_qmlimportscanner):
        # setting up mocks
        jar_dir = "tmp/jar/PySide6/jar"
        mock_extract_jar.return_value = Path(jar_dir)
        mock_qtlibs.return_value = self.pyside_wheel / "PySide6/Qt/lib"
        mock_extraqtmodules.return_value = ['Qml', 'Network', 'QmlModels', 'OpenGL']
        mock_recipes_exist.return_value = True
        jars, init_classes = ["/tmp/jar/PySide6/jar/Qt6Android.jar",
                              "/tmp/jar/PySide6/jar/Qt6AndroidBindings.jar",
                              "/tmp/jar/PySide6/jar/Qt6AndroidNetworkInformationBackend.jar",
                              "/tmp/jar/PySide6/jar/Qt6AndroidNetwork.jar"], []
        mock_find_jars.return_value = jars, init_classes
        dependent_plugins = ["platforms_qtforandroid",
                             "platforminputcontexts_qtvirtualkeyboardplugin",
                             "iconengines_qsvgicon"]
        mock_local_libs.return_value = [], dependent_plugins
        mock_qmlimportscanner.return_value = ["QtQuick"]

        self.android_deploy.main(name="android_app", shiboken_wheel=self.shiboken_wheel,
                                 pyside_wheel=self.pyside_wheel, ndk_path=self.ndk_path,
                                 init=True, force=True, keep_deployment_files=True)

        self.assertEqual(mock_extract_jar.call_count, 1)
        self.assertEqual(mock_qtlibs.call_count, 1)
        self.assertEqual(mock_extraqtmodules.call_count, 1)
        self.assertEqual(mock_recipes_exist.call_count, 1)
        self.assertEqual(mock_find_jars.call_count, 1)
        self.assertEqual(mock_qmlimportscanner.call_count, 1)
        self.assertTrue(self.config_file.exists())
        self.assertTrue(self.buildozer_config_file.exists())

        config_obj = self.deploy_lib.BaseConfig(config_file=self.config_file)
        expected_modules = {"Quick", "Core", "Gui", "Network", "Qml", "QmlModels", "OpenGL"}
        obtained_modules = set(config_obj.get_value("qt", "modules").split(","))
        self.assertEqual(obtained_modules, expected_modules)
        expected_local_libs = ""
        self.assertEqual(config_obj.get_value("buildozer", "local_libs"),
                         expected_local_libs)
        expected_qt_plugins = set(dependent_plugins)
        obtained_qt_plugins = set(config_obj.get_value("android", "plugins").split(","))
        self.assertEqual(expected_qt_plugins, obtained_qt_plugins)

        # test buildozer config file contents
        buildozer_config_obj = self.deploy_lib.BaseConfig(config_file=self.buildozer_config_file)
        obtained_jars = set(buildozer_config_obj.get_value("app", "android.add_jars").split(','))
        expected_jars = set(jars)
        self.assertEqual(obtained_jars, expected_jars)
        obtained_extra_args = buildozer_config_obj.get_value("app", "p4a.extra_args")
        extra_args_patrn = re.compile("--qt-libs=(?P<modules>.*) --load-local-libs="
                                      "(?P<local_libs>.*) --init-classes=(?P<init_classes>.*)")
        match = extra_args_patrn.search(obtained_extra_args)
        obtained_modules = match.group("modules").split(',')
        obtained_local_libs = match.group("local_libs")
        obtained_init_classes = match.group("init_classes")
        self.assertEqual(set(obtained_modules), expected_modules)
        self.assertEqual(obtained_local_libs, expected_local_libs)
        self.assertEqual(obtained_init_classes, '')

        self.config_file.unlink()
        self.buildozer_config_file.unlink()


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

import PySide6


# Note:
# "from PySide6 import *" can only be used at module level.
# It is also really not recommended to use. But for testing,
# the "__all__" variable is a great feature!
class AllModulesImportTest(unittest.TestCase):
    def testAllModulesCanImport(self):
        # would also work: exec("from PySide6 import *")
        for name in PySide6.__all__:
            exec(f"import PySide6.{name}")

    def testAllReappearsAfterDel(self):
        # This is the only incompatibility that remains:
        # After __all__ is deleted, it will re-appear.
        PySide6.__all__ = 42
        self.assertEqual(PySide6.__dict__["__all__"], 42)
        del PySide6.__all__
        self.assertTrue(PySide6.__dict__["__all__"])
        self.assertNotEqual(PySide6.__dict__["__all__"], 42)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QProcess'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QProcess


class TestQProcess (unittest.TestCase):
    def testStartDetached(self):
        value, pid = QProcess.startDetached("dir", [], os.getcwd())
        self.assertTrue(isinstance(value, bool))
        self.assertTrue(isinstance(pid, int))

    def testPid(self):
        p = QProcess()
        p.start("dir", [])
        p.waitForStarted()
        pid = p.processId()
        # We can't test the pid method result because it returns 0 when the
        # process isn't running
        if p.state() == QProcess.ProcessState.Running:
            self.assertNotEqual(pid, 0)
            p.waitForFinished()
        else:
            print("PROCESS ALREADY DEAD :-/")


if __name__ == '__main__':
    unittest.main()



import inspect
import sys

"""Helpers for determining base classes by importing the class.
When passed something like:
 PySide6.QtCore.QStateMachine.SignalEvent
try to import the underlying module and return a
handle to the object. In a loop, import
  PySide6.QtCore.QStateMachine.SignalEvent
  PySide6.QtCore.QStateMachine
  PySide6.QtCore
until the import succeeds and walk up the attributes
to obtain the object."""


TEST_DRIVER_USAGE = """Usage: import_inheritance.py class_name [current_module]

Example:
python import_inheritance.py PySide6.QtWidgets.QWizard PySide6.QtWidgets
"""


class InheritanceException(Exception):
    pass


def _importClassOrModule(name):
    components = name.split('.')
    for i in range(len(components), 0, -1):
        importPath = '.'.join(components[: i])
        try:
            __import__(importPath)
        except ImportError:
            continue
        if i == len(components):
            return sys.modules[importPath]
        remaining = components[i:]
        cls = sys.modules[importPath]
        for component in remaining:
            try:
                cls = getattr(cls, component)
            except Exception:  # No such attribute
                return None
        return cls
    return None


def _import_class_or_module(name, currmodule):
    """
    Import a class using its fully-qualified *name*.
    """
    todoc = _importClassOrModule(name)
    if not todoc and currmodule is not None:
        todoc = _importClassOrModule(f"{currmodule}.{name}")
    if not todoc:
        moduleStr = f'(module {currmodule})' if currmodule else ''
        raise InheritanceException(f'Could not import class {name} specified for '
                                   f'inheritance diagram {moduleStr}.')
    if inspect.isclass(todoc):
        return [todoc]
    elif inspect.ismodule(todoc):
        classes = []
        for cls in todoc.__dict__.values():
            if inspect.isclass(cls) and cls.__module__ == todoc.__name__:
                classes.append(cls)
        return classes
    raise InheritanceException(f'{name} specified for inheritance diagram is '
                               'not a class or module')


def _import_classes(class_names, currmodule):
    """Import a list of classes."""
    classes = []
    for name in class_names:
        classes.extend(_import_class_or_module(name, currmodule))
    return classes


def _class_name(cls, parts=0):
    """Given a class object, return a fully-qualified name.

    This works for things I've tested in matplotlib so far, but may not be
    completely general.
    """
    module = cls.__module__
    if module == '__builtin__':
        fullname = cls.__name__
    else:
        fullname = f"{module}.{cls.__qualname__}"
    if parts == 0:
        return fullname
    name_parts = fullname.split('.')
    return '.'.join(name_parts[-parts:])


def _class_info(classes, builtins=None, show_builtins=False, parts=0):
    """Return name and bases for all classes that are ancestors of
    *classes*.

    *parts* gives the number of dotted name parts that is removed from the
    displayed node names.
    """
    all_classes = {}
    builtins_list = builtins.values() if builtins else []

    def recurse(cls):
        if not show_builtins and cls in builtins_list:
            return

        nodename = _class_name(cls, parts)
        fullname = _class_name(cls, 0)

        baselist = []
        all_classes[cls] = (nodename, fullname, baselist)
        for base in cls.__bases__:
            if not show_builtins and base in builtins_list:
                continue
            if base.__name__ == "Object" and base.__module__ == "Shiboken":
                continue
            baselist.append(_class_name(base, parts))
            if base not in all_classes:
                recurse(base)

    for cls in classes:
        recurse(cls)

    return list(all_classes.values())


def get_inheritance_entries_by_import(class_names, currmodule,
                                      builtins=None,
                                      show_builtins=False, parts=0):
    classes = _import_classes(class_names, currmodule)
    class_info = _class_info(classes, builtins, show_builtins, parts)
    if not class_info:
        raise InheritanceException('No classes found for '
                                   'inheritance diagram')
    return class_info


if __name__ == "__main__":
    module = None
    if len(sys.argv) < 2:
        print(TEST_DRIVER_USAGE)
        sys.exit(-1)
    class_name = sys.argv[1]
    if len(sys.argv) >= 3:
        module = sys.argv[2]
    entries = get_inheritance_entries_by_import([class_name], module, None,
                                                False, 2)
    for e in entries:
        print(e)



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QLineF'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QLineF, QPointF


class TestQLineF (unittest.TestCase):
    def testIntersect(self):
        l1 = QLineF(0, 0, 1, 0)
        l2 = QLineF(1, -1, 1, 1)
        tuple_ = l1.intersects(l2)
        self.assertEqual(tuple, tuple_.__class__)
        (value, p) = tuple_
        self.assertEqual(QLineF.IntersectionType.BoundedIntersection, value)
        self.assertEqual(QPointF(1, 0), p)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2017 Klaralvdalens Datakonsult AB (KDAB).
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import QComboBox
from PySide6.QtCore import Signal, Slot
from PySide6.QtPdfWidgets import QPdfView


ZOOM_LEVELS = ["Fit Width", "Fit Page", "12%", "25%", "33%", "50%", "66%",
               "75%", "100%", "125%", "150%", "200%", "400%"]


class ZoomSelector(QComboBox):
    zoomModeChanged = Signal(QPdfView.ZoomMode)
    zoomFactorChanged = Signal(float)

    def __init__(self, parent):
        super().__init__(parent)
        self.setEditable(True)

        for z in ZOOM_LEVELS:
            self.addItem(z)

        self.currentTextChanged.connect(self.onCurrentTextChanged)
        self.lineEdit().editingFinished.connect(self._editingFinished)

    @Slot()
    def _editingFinished(self):
        self.onCurrentTextChanged(self.lineEdit().text())

    @Slot(float)
    def setZoomFactor(self, zoomFactor):
        z = int(100 * zoomFactor)
        self.setCurrentText(f"{z}%")

    @Slot()
    def reset(self):
        self.setCurrentIndex(8)  # 100%

    @Slot(str)
    def onCurrentTextChanged(self, text):
        if text == "Fit Width":
            self.zoomModeChanged.emit(QPdfView.ZoomMode.FitToWidth)
        elif text == "Fit Page":
            self.zoomModeChanged.emit(QPdfView.ZoomMode.FitInView)
        else:
            factor = 1.0
            withoutPercent = text.replace('%', '')
            zoomLevel = int(withoutPercent)
            if zoomLevel:
                factor = zoomLevel / 100.0

            self.zoomModeChanged.emit(QPdfView.ZoomMode.Custom)
            self.zoomFactorChanged.emit(factor)



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations
import sys
import logging
import zipfile
from dataclasses import dataclass
from pathlib import Path
from zipfile import ZipFile

from jinja2 import Environment, FileSystemLoader

from .. import run_command


@dataclass
class AndroidData:
    """
    Dataclass to store all the Android data obtained through cli
    """
    wheel_pyside: Path
    wheel_shiboken: Path
    ndk_path: Path
    sdk_path: Path


def create_recipe(version: str, component: str, wheel_path: str, generated_files_path: Path,
                  qt_modules: list[str] = None, local_libs: list[str] = None,
                  plugins: list[str] = None):
    '''
    Create python_for_android recipe for PySide6 and shiboken6
    '''
    qt_plugins = []
    if plugins:
        # split plugins based on category
        for plugin in plugins:
            plugin_category, plugin_name = plugin.split('_', 1)
            qt_plugins.append((plugin_category, plugin_name))

    qt_local_libs = []
    if local_libs:
        qt_local_libs = [local_lib for local_lib in local_libs if local_lib.startswith("Qt6")]

    rcp_tmpl_path = Path(__file__).parent / "recipes" / f"{component}"
    environment = Environment(loader=FileSystemLoader(rcp_tmpl_path))
    template = environment.get_template("__init__.tmpl.py")
    content = template.render(
        version=version,
        wheel_path=wheel_path,
        qt_modules=qt_modules,
        qt_local_libs=qt_local_libs,
        qt_plugins=qt_plugins
    )

    recipe_path = generated_files_path / "recipes" / f"{component}"
    recipe_path.mkdir(parents=True, exist_ok=True)
    logging.info(f"[DEPLOY] Writing {component} recipe into {str(recipe_path)}")
    with open(recipe_path / "__init__.py", mode="w", encoding="utf-8") as recipe:
        recipe.write(content)


def extract_and_copy_jar(wheel_path: Path, generated_files_path: Path) -> str:
    '''
    extracts the PySide6 wheel and copies the 'jar' folder to 'generated_files_path'.
    These .jar files are added to the buildozer.spec file to be used later by buildozer
    '''
    jar_path = generated_files_path / "jar"
    jar_path.mkdir(parents=True, exist_ok=True)
    archive = ZipFile(wheel_path)
    jar_files = [file for file in archive.namelist() if file.startswith("PySide6/jar")]
    for file in jar_files:
        archive.extract(file, jar_path)
    return (jar_path / "PySide6" / "jar").resolve() if jar_files else None


def get_wheel_android_arch(wheel: Path):
    '''
    Get android architecture from wheel
    '''
    supported_archs = ["aarch64", "armv7a", "i686", "x86_64"]
    for arch in supported_archs:
        if arch in wheel.stem:
            return arch

    return None


def get_llvm_readobj(ndk_path: Path) -> Path:
    '''
    Return the path to llvm_readobj from the Android Ndk
    '''
    # TODO: Requires change if Windows platform supports Android Deployment or if we
    # support host other than linux-x86_64
    return (ndk_path / f"toolchains/llvm/prebuilt/{sys.platform}-x86_64/bin/llvm-readobj")


def find_lib_dependencies(llvm_readobj: Path, lib_path: Path, used_dependencies: set[str] = None,
                          dry_run: bool = False):
    """
    Find all the Qt dependencies of a library using llvm_readobj
    """
    if lib_path.name in used_dependencies:
        return

    used_dependencies.add(lib_path.name)

    command = [str(llvm_readobj), "--needed-libs", str(lib_path)]

    # even if dry_run is given, we need to run the actual command to see all the dependencies
    # for which llvm-readelf is run.
    if dry_run:
        _, output = run_command(command=command, dry_run=dry_run, fetch_output=True)
    _, output = run_command(command=command, dry_run=False, fetch_output=True)

    dependencies = set()
    neededlibraries_found = False
    for line in output.splitlines():
        line = line.decode("utf-8").lstrip()
        if line.startswith("NeededLibraries") and not neededlibraries_found:
            neededlibraries_found = True
        if neededlibraries_found and line.startswith("libQt"):
            dependencies.add(line)
            used_dependencies.add(line)
            dependent_lib_path = lib_path.parent / line
            find_lib_dependencies(llvm_readobj, dependent_lib_path, used_dependencies, dry_run)

    if dependencies:
        logging.info(f"[DEPLOY] Following dependencies found for {lib_path.stem}: {dependencies}")
    else:
        logging.info(f"[DEPLOY] No Qt dependencies found for {lib_path.stem}")


def find_qtlibs_in_wheel(wheel_pyside: Path):
    """
    Find the path to Qt/lib folder inside the wheel.
    """
    archive = ZipFile(wheel_pyside)
    qt_libs_path = wheel_pyside / "PySide6/Qt/lib"
    qt_libs_path = zipfile.Path(archive, at=qt_libs_path)
    if not qt_libs_path.exists():
        for file in archive.namelist():
            # the dependency files are inside the libs folder
            if file.endswith("android-dependencies.xml"):
                qt_libs_path = zipfile.Path(archive, at=file).parent
                # all dependency files are in the same path
                break

    if not qt_libs_path:
        raise FileNotFoundError("[DEPLOY] Unable to find Qt libs folder inside the wheel")

    return qt_libs_path



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

from functools import reduce
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from minimal import ListUser, Val, Obj, StdIntList


class ExtListUser(ListUser):
    def __init__(self):
        super().__init__()
        self._stdIntList = StdIntList()
        self._stdIntList.append(1)
        self._stdIntList.append(2)

    def createIntList(self, num):
        return list(range(0, num * 2, 2))

    def sumIntList(self, intList):
        return sum(intList) * 2

    def createMinBoolList(self, mb1, mb2):
        return [not mb1, not mb2]

    def oredMinBoolList(self, minBoolList):
        return not reduce(lambda a, b: a | b, minBoolList)

    def createValList(self, num):
        return [Val(i) for i in range(0, num * 2, 2)]

    def sumValList(self, valList):
        return sum([val.valId() for val in valList]) * 2

    def createObjList(self, o1, o2):
        o1.setObjId(o1.objId() * 2)
        o2.setObjId(o2.objId() * 2)
        return [o1, o2]

    def sumObjList(self, objList):
        return sum([obj.objId() for obj in objList]) * 2

    def createListOfIntLists(self, num):
        return [self.createIntList(num)] * 4

    def sumListOfIntLists(self, intListList):
        return sum([sum(line) for line in intListList]) * 2

    def returnIntListByPtr(self):
        return self._stdIntList


class IntListConversionTest(unittest.TestCase):

    def testCreateIntList(self):
        num = 4
        lu = ListUser()
        lst = lu.createIntList(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), int)
        self.assertEqual(lst, list(range(num)))
        lst = lu.callCreateIntList(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), int)
        self.assertEqual(lst, list(range(num)))

    def testCreateIntListFromExtendedClass(self):
        lu = ExtListUser()
        num = 4
        lst = lu.createIntList(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), int)
        self.assertEqual(lst, list(range(0, num * 2, 2)))
        lst = lu.callCreateIntList(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), int)
        self.assertEqual(lst, list(range(0, num * 2, 2)))

    def testSumIntList(self):
        lu = ListUser()
        lst = range(4)
        expected = sum(lst)
        self.assertEqual(lu.sumIntList(lst), expected)
        self.assertEqual(lu.callSumIntList(lst), expected)
        self.assertEqual(lu.sumIntListDefaultParam(lst), expected)
        self.assertEqual(lu.sumIntListDefaultParamConstRef(lst), expected)
        # PYSIDE-2454: Check container default parameters (1,2,3)
        self.assertEqual(lu.sumIntListDefaultParam(), 6)
        self.assertEqual(lu.sumIntListDefaultParamConstRef(), 6)

    def testSumIntListFromExtendedClass(self):
        lu = ExtListUser()
        lst = range(4)
        self.assertEqual(lu.sumIntList(lst), sum(lst) * 2)
        self.assertEqual(lu.callSumIntList(lst), sum(lst) * 2)


class MinBoolListConversionTest(unittest.TestCase):

    def testCreateMinBoolList(self):
        lu = ListUser()
        lst = lu.createMinBoolList(True, False)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), 2)
        for i in lst:
            self.assertEqual(type(i), bool)
        self.assertEqual(lst, [True, False])

        lst = lu.callCreateMinBoolList(False, True)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), 2)
        for i in lst:
            self.assertEqual(type(i), bool)
        self.assertEqual(lst, [False, True])

    def testCreateMinBoolListFromExtendedClass(self):
        lu = ExtListUser()
        lst = lu.createMinBoolList(True, False)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), 2)
        for i in lst:
            self.assertEqual(type(i), bool)
        self.assertEqual(lst, [False, True])

        lst = lu.callCreateMinBoolList(False, True)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), 2)
        for i in lst:
            self.assertEqual(type(i), bool)
        self.assertEqual(lst, [True, False])

    def testOredMinBoolList(self):
        lu = ListUser()
        lst = [False, False, True]
        self.assertTrue(lu.oredMinBoolList(lst))
        self.assertTrue(lu.callOredMinBoolList(lst))
        lst = [False, False, False]
        self.assertFalse(lu.oredMinBoolList(lst))
        self.assertFalse(lu.callOredMinBoolList(lst))

    def testOredMinBoolListFromExtendedClass(self):
        lu = ExtListUser()
        lst = [False, False, True]
        self.assertFalse(lu.oredMinBoolList(lst))
        self.assertFalse(lu.callOredMinBoolList(lst))
        lst = [False, False, False]
        self.assertTrue(lu.oredMinBoolList(lst))
        self.assertTrue(lu.callOredMinBoolList(lst))


class ValListConversionTest(unittest.TestCase):

    def testCreateValList(self):
        num = 4
        lu = ListUser()
        lst = lu.createValList(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), Val)
        self.assertEqual([val.valId() for val in lst], list(range(num)))
        lst = lu.callCreateValList(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), Val)
        self.assertEqual([val.valId() for val in lst], list(range(num)))

    def testCreateValListFromExtendedClass(self):
        lu = ExtListUser()
        num = 4
        lst = lu.createValList(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), Val)
        self.assertEqual([val.valId() for val in lst], list(range(0, num * 2, 2)))
        lst = lu.callCreateValList(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), Val)
        self.assertEqual([val.valId() for val in lst], list(range(0, num * 2, 2)))

    def testSumValList(self):
        lu = ListUser()
        lst = [Val(i) for i in range(4)]
        self.assertEqual(lu.sumValList(lst), sum([val.valId() for val in lst]))
        self.assertEqual(lu.callSumValList(lst), sum([val.valId() for val in lst]))

    def testSumValListFromExtendedClass(self):
        lu = ExtListUser()
        lst = [Val(i) for i in range(4)]
        self.assertEqual(lu.sumValList(lst), sum([val.valId() for val in lst]) * 2)
        self.assertEqual(lu.callSumValList(lst), sum([val.valId() for val in lst]) * 2)


class ObjListConversionTest(unittest.TestCase):

    def testCreateObjList(self):
        o1 = Obj(1)
        o2 = Obj(2)
        lu = ListUser()
        lst = lu.createObjList(o1, o2)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), 2)
        for i in lst:
            self.assertEqual(type(i), Obj)
        self.assertEqual(lst, [o1, o2])
        self.assertEqual([obj.objId() for obj in lst], [1, 2])

        lst = lu.callCreateObjList(o1, o2)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), 2)
        for i in lst:
            self.assertEqual(type(i), Obj)
        self.assertEqual(lst, [o1, o2])
        self.assertEqual([obj.objId() for obj in lst], [1, 2])

    def testCreateObjListFromExtendedClass(self):
        o1 = Obj(1)
        o2 = Obj(2)
        lu = ExtListUser()
        lst = lu.createObjList(o1, o2)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), 2)
        for i in lst:
            self.assertEqual(type(i), Obj)
        self.assertEqual(lst, [o1, o2])
        self.assertEqual([obj.objId() for obj in lst], [2, 4])

        lst = lu.callCreateObjList(o1, o2)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), 2)
        for i in lst:
            self.assertEqual(type(i), Obj)
        self.assertEqual(lst, [o1, o2])
        self.assertEqual([obj.objId() for obj in lst], [4, 8])

    def testSumObjList(self):
        lu = ListUser()
        lst = [Obj(i) for i in list(range(4))]
        self.assertEqual(lu.sumObjList(lst), sum([obj.objId() for obj in lst]))
        self.assertEqual(lu.callSumObjList(lst), sum([obj.objId() for obj in lst]))

    def testSumObjListFromExtendedClass(self):
        lu = ExtListUser()
        lst = [Obj(i) for i in list(range(4))]
        self.assertEqual(lu.sumObjList(lst), sum([obj.objId() for obj in lst]) * 2)
        self.assertEqual(lu.callSumObjList(lst), sum([obj.objId() for obj in lst]) * 2)


class ListOfIntListConversionTest(unittest.TestCase):

    def testCreateListOfIntLists(self):
        num = 4
        lu = ListUser()
        lst = lu.createListOfIntLists(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), list)
            self.assertEqual(i, list(range(num)))
            for j in i:
                self.assertEqual(type(j), int)
        self.assertEqual(lst, [list(range(num))] * 4)

    def testCreateListOfIntListsFromExtendedClass(self):
        num = 4
        lu = ExtListUser()
        lst = lu.createListOfIntLists(num)
        self.assertEqual(type(lst), list)
        self.assertEqual(len(lst), num)
        for i in lst:
            self.assertEqual(type(i), list)
            self.assertEqual(i, list(range(0, num * 2, 2)))
            for j in i:
                self.assertEqual(type(j), int)
        self.assertEqual(lst, [list(range(0, num * 2, 2))] * 4)

    def testSumListIntLists(self):
        lu = ListUser()
        lst = [range(4)] * 4
        self.assertEqual(lu.sumListOfIntLists(lst), sum([sum(line) for line in [range(4)] * 4]))
        self.assertEqual(lu.callSumListOfIntLists(lst), sum([sum(line) for line in [range(4)] * 4]))

    def testSumListOfIntListsFromExtendedClass(self):
        lu = ExtListUser()
        lst = [range(4)] * 4
        self.assertEqual(lu.sumListOfIntLists(lst),
                         sum([sum(line) for line in [range(4)] * 4]) * 2)
        self.assertEqual(lu.callSumListOfIntLists(lst),
                         sum([sum(line) for line in [range(4)] * 4]) * 2)

    def testOpaqueContainer(self):
        lu = ListUser()

        # Set via Python
        python_list = [1, 2]
        lu.setStdIntList(python_list)
        self.assertEqual(len(lu.m_stdIntList), 2)
        self.assertEqual(lu.m_stdIntList[0], 1)
        self.assertEqual(lu.m_stdIntList[1], 2)

        # Set via C++
        cpp_list = StdIntList()
        cpp_list.append(3)
        cpp_list.append(4)
        lu.setStdIntList(cpp_list)
        self.assertEqual(len(lu.m_stdIntList), 2)
        self.assertEqual(lu.m_stdIntList[0], 3)
        self.assertEqual(lu.m_stdIntList[1], 4)

        # Access field directly via reference
        lu.m_stdIntList.append(5)
        self.assertEqual(len(lu.m_stdIntList), 3)
        self.assertEqual(lu.m_stdIntList[2], 5)

        # Access list via getter
        il = lu.getIntList()
        il.append(6)
        self.assertEqual(len(lu.m_stdIntList), 4)
        self.assertEqual(lu.m_stdIntList[3], 6)

        # Access a const list via getter and verify that it cannot be modified
        const_l = lu.getConstIntList()
        self.assertEqual(len(const_l), 4)
        self.assertRaises(TypeError, const_l.append, 6)

    def testListByPtrOpaque(self):
        """Test a function taking C++ list by pointer for which an opaque
           container exists."""
        lu = ListUser()
        python_list = [1, 2]
        self.assertEqual(lu.modifyIntListPtr(python_list), 2)

        # Pass an opaque container and verify whether it is modified by C++
        cpp_list = StdIntList()
        cpp_list.append(1)
        cpp_list.append(2)
        self.assertEqual(lu.modifyIntListPtr(cpp_list), 2)
        self.assertEqual(len(cpp_list), 3)

    def testListByPtr(self):
        """Test a function taking C++ list by pointer for which no opaque
           container exists."""
        lu = ListUser()
        python_list = [1.1, 22.2]
        self.assertEqual(lu.modifyDoubleListPtr(python_list), 2)

    def testReturnListByPtr(self):
        """Test that a virtual function returning a list by pointer can be
           reimplemented by a Python function returning an opaque container."""
        lu = ExtListUser()
        size = lu.callReturnIntListByPtr()  # Call virtual from C++
        self.assertEqual(size, 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for __hash__'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ObjectType, Str


class HashableTest(unittest.TestCase):

    def testStrHash(self):
        h = {}
        s = Str("Hi")
        h[s] = 2
        self.assertTrue(h.get(s), 2)

    def testObjectTypeHash(self):
        h = {}
        o = ObjectType()
        h[o] = 2
        self.assertTrue(h.get(o), 2)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Qt, qVersion, qFuzzyCompare
from PySide6.QtGui import QGuiApplication, QFontDatabase
from PySide6.QtWidgets import (QDialog, QDialogButtonBox,
                               QPlainTextEdit, QVBoxLayout)


def _format_font(font):
    family = font.family()
    size = font.pointSizeF()
    return f"{family}, {size}pt"


class FontInfoDialog(QDialog):

    def __init__(self, parent):
        super().__init__(parent)
        self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
        main_layout = QVBoxLayout(self)
        text_edit = QPlainTextEdit(self.text(), self)
        text_edit.setReadOnly(True)
        text_edit.setFont(QFontDatabase.systemFont(QFontDatabase.FixedFont))
        main_layout.addWidget(text_edit)
        button_box = QDialogButtonBox(QDialogButtonBox.StandardButton.Close, self)
        button_box.rejected.connect(self.reject)
        main_layout.addWidget(button_box)

    def text(self):
        default_font = QFontDatabase.systemFont(QFontDatabase.GeneralFont)
        fixed_font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
        title_font = QFontDatabase.systemFont(QFontDatabase.TitleFont)
        smallest_readable_font = QFontDatabase.systemFont(QFontDatabase.SmallestReadableFont)

        v = qVersion()
        platform = QGuiApplication.platformName()
        dpi = self.logicalDpiX()
        dpr = self.devicePixelRatio()
        text = f"Qt {v} on {platform}, {dpi}DPI"
        if not qFuzzyCompare(dpr, float(1)):
            text += f", device pixel ratio: {dpr}"
        text += ("\n\nDefault font : " + _format_font(default_font)
                 + "\nFixed font : " + _format_font(fixed_font)
                 + "\nTitle font : " + _format_font(title_font)
                 + "\nSmallest font: " + _format_font(smallest_readable_font))
        return text



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QTimer
from PySide6.QtGui import QActionGroup
from PySide6.QtWidgets import QApplication
from PySide6.QtUiTools import QUiLoader


class Window:
    def __init__(self):
        loader = QUiLoader()
        filePath = os.path.join(os.path.dirname(__file__), 'bug_426.ui')
        self.widget = loader.load(filePath)
        self.group = QActionGroup(self.widget)
        self.widget.show()
        QTimer.singleShot(0, self.widget.close)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    win = Window()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test cases related to QGraphicsItem and subclasses'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QGraphicsObject, QGraphicsWidget
from PySide6.QtCore import QRectF

from helper.usesqapplication import UsesQApplication


class GObjA(QGraphicsObject):
    def paint(self, *args):
        pass

    def boundingRect(self):
        return QRectF()

    def itemChange(self, *args):
        return QGraphicsObject.itemChange(self, *args)


class GObjB(QGraphicsObject):
    def paint(self, *args):
        pass

    def boundingRect(self):
        return QRectF()


class QGraphicsObjectReimpl(UsesQApplication):
    '''Test case for reimplementing QGraphicsObject'''

    def testReimplementationTypes(self):
        w = QGraphicsWidget()

        # PYSIDE-86:
        #   This case failed because GObjA was reimplementing
        #   the method itemChange() from QGraphicsItem,
        #   and then the QVariant was not associated with
        #   a QGraphicsItem but a QObjectItem because the base
        #   class was a QObject.
        #   See also PYSIDE-1887, PYSIDE-3069
        gobjA = GObjA()
        gobjA.setParentItem(w)
        print(gobjA.parentItem())
        self.assertIs(type(w), type(gobjA.parentItem()))

        # PYSIDE-3115: QVariant conversion of the parent
        # (Python class inheriting QGraphicsObject).
        parentA = GObjA()
        gobjA = GObjA()
        gobjA.setParentItem(parentA)
        self.assertIs(type(parentA), type(gobjA.parentItem()))

        gobjB = GObjB()
        gobjB.setParentItem(w)
        self.assertIs(type(w), type(gobjB.parentItem()))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QTimer, QObject, Signal


class TestDestroySignal(unittest.TestCase):
    def onObjectDestroyed(self, timer):
        self.assertTrue(isinstance(timer, QObject))
        self._destroyed = True

    def testSignal(self):
        self._destroyed = False
        t = QTimer()
        t.destroyed[QObject].connect(self.onObjectDestroyed)
        del t
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        # PYSIDE-535: Why do I need to do it twice, here?
        gc.collect()
        self.assertTrue(self._destroyed)

    def testWithParent(self):
        self._destroyed = False
        p = QTimer()
        t = QTimer(p)
        t.destroyed[QObject].connect(self.onObjectDestroyed)
        del p
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        # PYSIDE-535: Why do I need to do it twice, here?
        gc.collect()
        self.assertTrue(self._destroyed)


class Foo(QObject):
    s = Signal(int)

    def __init__(self):
        QObject.__init__(self)
        sys.stderr.write(f"__init__ {id(self):x}\n")

    def __del__(self):
        sys.stderr.write(f"__del__  {id(self):x}\n")

    def send(self, i):
        self.s.emit(i)


# PYSIDE-2201/2328: This crashed until we introduced a weak reference.
class TestDestroyNoConnect(unittest.TestCase):

    def testSignalDestroyedMissingReference(self):
        # This works since it has one reference more to Foo
        Foo().send(43)
        # This crashed because we have no reference in the signal.
        with self.assertRaises(RuntimeError):
            Foo().s.emit(44)

    def testSignalDestroyedinConnect(self):
        # PYSIDE-2328: Connect to signal of temporary
        with self.assertRaises(TypeError):
            Foo().s.connect(None)


if __name__ == '__main__':
    unittest.main()



# Converted from webauthdialog.cpp

# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from ui_webauthdialog import Ui_WebAuthDialog

from PySide6.QtWidgets import (QDialog, QVBoxLayout, QButtonGroup,
                               QScrollArea, QWidget, QDialogButtonBox,
                               QSizePolicy, QRadioButton)
from PySide6.QtCore import Qt
from PySide6.QtWebEngineCore import QWebEngineWebAuthUxRequest


class WebAuthDialog(QDialog):

    def __init__(self, request, parent=None):
        super().__init__(parent)

        self.uxRequest = request
        self.uiWebAuthDialog = Ui_WebAuthDialog()
        self.uiWebAuthDialog.setupUi(self)

        self.button_group = QButtonGroup(self)
        self.button_group.setExclusive(True)

        self.scroll_area = QScrollArea(self)
        self.select_account_widget = QWidget(self)
        self.scroll_area.setWidget(self.select_account_widget)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.select_account_widget.resize(400, 150)

        self.select_account_layout = QVBoxLayout(self.select_account_widget)
        self.uiWebAuthDialog.m_mainVerticalLayout.addWidget(self.scroll_area)
        self.select_account_layout.setAlignment(Qt.AlignTop)

        self.update_display()

        self.uiWebAuthDialog.buttonBox.rejected.connect(self.onCancelRequest)
        self.uiWebAuthDialog.buttonBox.accepted.connect(self.onAcceptRequest)

        button = self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Retry)
        button.clicked.connect(self.onRetry)
        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)

    def __del__(self):
        for button in self.button_group.buttons():
            button.deleteLater()

        if self.button_group:
            self.button_group.deleteLater()
            self.button_group = None

        if self.uiWebAuthDialog:
            del self.uiWebAuthDialog
            self.uiWebAuthDialog = None

        if self.scroll_area:
            self.scroll_area.deleteLater()
            self.scroll_area = None

    def update_display(self):
        state = self.uxRequest.state()
        if state == QWebEngineWebAuthUxRequest.WebAuthUxState.SelectAccount:
            self.setupSelectAccountUI()
        elif state == QWebEngineWebAuthUxRequest.WebAuthUxState.CollectPin:
            self.setupCollectPinUI()
        elif state == QWebEngineWebAuthUxRequest.WebAuthUxState.FinishTokenCollection:
            self.setupFinishCollectTokenUI()
        elif state == QWebEngineWebAuthUxRequest.WebAuthUxState.RequestFailed:
            self.setupErrorUI()

        self.adjustSize()

    def setupSelectAccountUI(self):
        self.uiWebAuthDialog.m_headingLabel.setText(self.tr("Choose a Passkey"))
        self.uiWebAuthDialog.m_description.setText(self.tr("Which passkey do you want to use for ")
                                                   + self.uxRequest.relyingPartyId()
                                                   + self.tr("? "))
        self.uiWebAuthDialog.m_pinGroupBox.setVisible(False)
        self.uiWebAuthDialog.m_mainVerticalLayout.removeWidget(self.uiWebAuthDialog.m_pinGroupBox)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Retry).setVisible(False)

        self.clearSelectAccountButtons()
        self.scroll_area.setVisible(True)
        self.select_account_widget.resize(self.width(), self.height())
        userNames = self.uxRequest.userNames()
        # Create radio buttons for each name
        for name in userNames:
            radioButton = QRadioButton(name)
            self.select_account_layout.addWidget(radioButton)
            self.button_group.addButton(radioButton)

        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Ok).setText(self.tr("Ok"))
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Ok).setVisible(True)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Cancel).setVisible(True)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Retry).setVisible(False)

    def setupFinishCollectTokenUI(self):

        self.clearSelectAccountButtons()
        self.uiWebAuthDialog.m_headingLabel.setText(self.tr("Use your security key with")
                                                    + self.uxRequest.relyingPartyId())
        self.uiWebAuthDialog.m_description.setText(
            self.tr("Touch your security key again to complete the request."))
        self.uiWebAuthDialog.m_pinGroupBox.setVisible(False)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Ok).setVisible(False)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Retry).setVisible(False)
        self.scroll_area.setVisible(False)

    def setupCollectPinUI(self):

        self.clearSelectAccountButtons()
        self.uiWebAuthDialog.m_mainVerticalLayout.addWidget(self.uiWebAuthDialog.m_pinGroupBox)
        self.uiWebAuthDialog.m_pinGroupBox.setVisible(True)
        self.uiWebAuthDialog.m_confirmPinLabel.setVisible(False)
        self.uiWebAuthDialog.m_confirmPinLineEdit.setVisible(False)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Ok).setText(self.tr("Next"))
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Ok).setVisible(True)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Cancel).setVisible(True)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Retry).setVisible(False)
        self.scroll_area.setVisible(False)

        pinRequestInfo = self.uxRequest.pinRequest()

        if pinRequestInfo.reason == QWebEngineWebAuthUxRequest.PinEntryReason.Challenge:
            self.uiWebAuthDialog.m_headingLabel.setText(self.tr("PIN Required"))
            self.uiWebAuthDialog.m_description.setText(
                self.tr("Enter the PIN for your security key"))
            self.uiWebAuthDialog.m_confirmPinLabel.setVisible(False)
            self.uiWebAuthDialog.m_confirmPinLineEdit.setVisible(False)
        else:
            if pinRequestInfo.reason == QWebEngineWebAuthUxRequest.PinEntryReason.Set:
                self.uiWebAuthDialog.m_headingLabel.setText(self.tr("New PIN Required"))
                self.uiWebAuthDialog.m_description.setText(
                    self.tr("Set new PIN for your security key"))
            else:
                self.uiWebAuthDialog.m_headingLabel.setText(self.tr("Change PIN Required"))
                self.uiWebAuthDialog.m_description.setText(
                    self.tr("Change PIN for your security key"))

            self.uiWebAuthDialog.m_confirmPinLabel.setVisible(True)
            self.uiWebAuthDialog.m_confirmPinLineEdit.setVisible(True)

        errorDetails = ""

        if pinRequestInfo.error == QWebEngineWebAuthUxRequest.PinEntryError.InternalUvLocked:
            errorDetails = self.tr("Internal User Verification Locked ")
        elif pinRequestInfo.error == QWebEngineWebAuthUxRequest.PinEntryError.WrongPin:
            errorDetails = self.tr("Wrong PIN")
        elif pinRequestInfo.error == QWebEngineWebAuthUxRequest.PinEntryError.TooShort:
            errorDetails = self.tr("Too Short")
        elif pinRequestInfo.error == QWebEngineWebAuthUxRequest.PinEntryError.InvalidCharacters:
            errorDetails = self.tr("Invalid Characters")
        elif pinRequestInfo.error == QWebEngineWebAuthUxRequest.PinEntryError.SameAsCurrentPin:
            errorDetails = self.tr("Same as current PIN")

        if errorDetails:
            errorDetails += f" {pinRequestInfo.remainingAttempts} attempts remaining"

        self.uiWebAuthDialog.m_pinEntryErrorLabel.setText(errorDetails)

    def onCancelRequest(self):

        self.uxRequest.cancel()

    def onAcceptRequest(self):

        state = self.uxRequest.state()
        if state == QWebEngineWebAuthUxRequest.WebAuthUxState.SelectAccount:
            if self.button_group.checkedButton():
                self.uxRequest.setSelectedAccount(self.button_group.checkedButton().text())
        elif state == QWebEngineWebAuthUxRequest.WebAuthUxState.CollectPin:
            self.uxRequest.setPin(self.uiWebAuthDialog.m_pinLineEdit.text())

    def setupErrorUI(self):

        self.clearSelectAccountButtons()
        error_description = ""
        error_heading = self.tr("Something went wrong")
        isVisibleRetry = False

        state = self.uxRequest.requestFailureReason()
        failure_reason = QWebEngineWebAuthUxRequest.RequestFailureReason

        if state == failure_reason.Timeout:
            error_description = self.tr("Request Timeout")
        elif state == failure_reason.KeyNotRegistered:
            error_description = self.tr("Key not registered")
        elif state == failure_reason.KeyAlreadyRegistered:
            error_description = self.tr("You already registered self device."
                                        "Try again with device")
            isVisibleRetry = True
        elif state == failure_reason.SoftPinBlock:
            error_description = self.tr(
                "The security key is locked because the wrong PIN was entered too many times."
                "To unlock it, remove and reinsert it.")
            isVisibleRetry = True
        elif state == failure_reason.HardPinBlock:
            error_description = self.tr(
                "The security key is locked because the wrong PIN was entered too many times."
                " Yo'll need to reset the security key.")
        elif state == failure_reason.AuthenticatorRemovedDuringPinEntry:
            error_description = self.tr(
                "Authenticator removed during verification. Please reinsert and try again")
        elif state == failure_reason.AuthenticatorMissingResidentKeys:
            error_description = self.tr("Authenticator doesn't have resident key support")
        elif state == failure_reason.AuthenticatorMissingUserVerification:
            error_description = self.tr("Authenticator missing user verification")
        elif state == failure_reason.AuthenticatorMissingLargeBlob:
            error_description = self.tr("Authenticator missing Large Blob support")
        elif state == failure_reason.NoCommonAlgorithms:
            error_description = self.tr("Authenticator missing Large Blob support")
        elif state == failure_reason.StorageFull:
            error_description = self.tr("Storage Full")
        elif state == failure_reason.UserConsentDenied:
            error_description = self.tr("User consent denied")
        elif state == failure_reason.WinUserCancelled:
            error_description = self.tr("User Cancelled Request")

        self.uiWebAuthDialog.m_headingLabel.setText(error_heading)
        self.uiWebAuthDialog.m_description.setText(error_description)
        self.uiWebAuthDialog.m_description.adjustSize()
        self.uiWebAuthDialog.m_pinGroupBox.setVisible(False)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Ok).setVisible(False)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Retry).setVisible(isVisibleRetry)
        if isVisibleRetry:
            self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Retry).setFocus()
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Cancel).setVisible(True)
        self.uiWebAuthDialog.buttonBox.button(QDialogButtonBox.Cancel).setText(self.tr("Close"))
        self.scroll_area.setVisible(False)

    def onRetry(self):
        self.uxRequest.retry()

    def clearSelectAccountButtons(self):
        buttons = self.button_group.buttons()

        for radio_button in buttons:
            self.select_account_layout.removeWidget(radio_button)
            self.button_group.removeButton(radio_button)
            radio_button.deleteLater()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtCore import QObject, Property, Signal
from PySide6.QtBluetooth import QBluetoothDeviceInfo


class DeviceInfo(QObject):

    device_changed = Signal()

    def __init__(self, d: QBluetoothDeviceInfo = None) -> None:
        super().__init__()
        self._device = d

    @Property(str, notify=device_changed)
    def device_name(self):
        return self._device.name()

    @Property(str, notify=device_changed)
    def device_address(self):
        if sys.platform == "darwin":
            return self._device.deviceUuid().toString()

        return self._device.address().toString()

    def get_device(self):
        return self._device

    def set_device(self, device):
        self._device = device
        self.device_changed.emit()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtCore import (QCoreApplication, QParallelAnimationGroup,  # noqa: E402
                            QSequentialAnimationGroup, QTimer)  # noqa: E402


class QAnimationGroupTest(unittest.TestCase):

    def testBasic(self):
        app = QCoreApplication([])

        panim = QParallelAnimationGroup()
        sanim = QSequentialAnimationGroup()
        panim.addAnimation(sanim)

        self.assertEqual(sanim.parent(), panim)

        panim.clear()

        self.assertRaises(RuntimeError, lambda: sanim.parent())

        QTimer.singleShot(100, app.quit)
        app.exec()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for python conversions types '''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
from datetime import date

from sample import SbkDate


class DateConversionTest(unittest.TestCase):

    def testConstructorWithDateObject(self):
        pyDate = date(2010, 12, 12)
        cDate = SbkDate(pyDate)
        self.assertTrue(cDate.day(), pyDate.day)
        self.assertTrue(cDate.month(), pyDate.month)
        self.assertTrue(cDate.year(), pyDate.year)

    def testToPythonFunction(self):
        cDate = SbkDate(2010, 12, 12)
        pyDate = cDate.toPython()
        self.assertTrue(cDate.day(), pyDate.day)
        self.assertTrue(cDate.month(), pyDate.month)
        self.assertTrue(cDate.year(), pyDate.year)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QDataStream'''

import os
import sys
import unittest

from pathlib import Path

sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths

init_test_paths(False)

from PySide6.QtCore import QBitArray, QByteArray, QIODevice, QDataStream, QDate, QTime, QDateTime


def create_bitarray(string):
    array = QBitArray(len(string))
    for i, char in enumerate(string):
        array.setBit(i, char != '0')
    return array


def serialize_bitarray(bit_array):
    buffer = QByteArray()
    stream = QDataStream(buffer, QIODevice.OpenModeFlag.WriteOnly)
    stream << bit_array
    return buffer


class QDataStreamWrite(unittest.TestCase):
    '''Test case for QDatastream write* functions'''

    def setUp(self):
        self.ba = QByteArray()
        self.read = QDataStream(self.ba, QIODevice.OpenModeFlag.ReadOnly)
        self.write = QDataStream(self.ba, QIODevice.OpenModeFlag.WriteOnly)

    def testWriteUInt8(self):
        '''QDataStream.writeUInt8 (accepting str of size 1)'''
        x = 0xFF
        self.write.writeUInt8(x)
        y = self.read.readUInt8()
        self.assertEqual(x, y)

        self.assertRaises(TypeError, self.write.writeUInt8, 'aaaaa')

    def testWriteInt8(self):
        '''QDataStream.writeInt8 (accepting str of size 1)'''
        x = 65
        self.write.writeInt8(chr(x))
        y = self.read.readInt8()
        self.assertEqual(x, y)

        self.assertRaises(TypeError, self.write.writeInt8, 'aaaaa')

    def testWriteUInt8Int(self):
        '''QDataStream.writeUInt8 (accepting integer)'''
        x = 0xFF
        self.write.writeUInt8(x)
        y = self.read.readUInt8()
        self.assertEqual(x, y)

    def testWriteInt8Int(self):
        '''QDataStream.writeInt8 (accepting integer)'''
        x = 65
        self.write.writeInt8(x)
        y = self.read.readInt8()
        self.assertEqual(x, y)

    def testWriteUInt16(self):
        '''QDataStream.writeUInt16'''
        x = 0x4423
        self.write.writeUInt16(x)
        y = int(self.read.readUInt16())
        self.assertEqual(x, y)

    def testWriteUInt32(self):
        '''QDataStream.writeUInt32'''
        x = 0xdeadbeef
        self.write.writeUInt32(x)
        y = int(self.read.readUInt32())
        self.assertEqual(x, y)


class QDataStreamShift(unittest.TestCase):
    '''Test case for << and >> operators'''

    def setUp(self):
        self.ba = QByteArray()
        self.stream = QDataStream(self.ba, QIODevice.OpenModeFlag.WriteOnly)
        self.read_stream = QDataStream(self.ba, QIODevice.OpenModeFlag.ReadOnly)

    def testQCharValid(self):
        '''QDataStream <<>> QChar - valid'''
        self.stream.writeQChar(42)

        res = self.read_stream.readQChar()
        self.assertEqual(res, chr(42))

    def testQCharNull(self):
        '''QDataStream <<>> QChar - null'''
        self.stream.writeQChar(None)

        res = self.read_stream.readQChar()
        self.assertEqual(res, '\x00')

    def testQByteArrayValid(self):
        '''QDataStream <<>> QByteArray - valid'''
        self.stream << QByteArray(bytes("hello", "UTF-8"))

        res = QByteArray()

        self.read_stream >> res
        self.assertEqual(res, QByteArray(bytes("hello", "UTF-8")))

    def testQByteArrayEmpty(self):
        '''QDataStream <<>> QByteArray - empty'''
        self.stream << QByteArray(bytes("", "UTF-8"))

        res = QByteArray()

        self.read_stream >> res
        self.assertEqual(res, QByteArray(bytes("", "UTF-8")))
        self.assertTrue(res.isEmpty())
        self.assertFalse(res.isNull())

    def testQByteArrayNull(self):
        '''QDataStream <<>> QByteArray - null'''
        self.stream << QByteArray()

        res = QByteArray()

        self.read_stream >> res
        self.assertEqual(res, QByteArray())
        self.assertTrue(res.isEmpty())
        self.assertTrue(res.isNull())

    def testQStringValid(self):
        '''QDataStream <<>> QString - valid'''
        self.stream.writeQString('Ka-boom')

        res = self.read_stream.readQString()
        self.assertEqual(res, 'Ka-boom')

    def testQStringEmpty(self):
        '''QDataStream <<>> QString - empty'''
        self.stream.writeQString('')

        res = self.read_stream.readQString()
        self.assertEqual(res, '')

    def testQStringNull(self):
        '''QDataStream <<>> QString - null'''
        self.stream.writeQString(None)

        res = self.read_stream.readQString()
        self.assertEqual(res, '')

    def testQBitArrayNull(self):
        '''QDataStream <<>> QBitArray - null'''
        self.stream << QBitArray()

        res = QBitArray()

        self.read_stream >> res
        self.assertEqual(res, QBitArray())

    def testQBitArrayValid(self):
        '''QDataStream <<>> QBitArray - valid'''
        self.stream << create_bitarray('01010101')

        res = QBitArray()

        self.read_stream >> res
        self.assertEqual(res, create_bitarray('01010101'))

    def testQDateNull(self):
        '''QDataStream <<>> QDate - null'''
        self.stream << QDate()

        res = QDate()

        self.read_stream >> res
        self.assertEqual(res, QDate())
        self.assertFalse(res.isValid())
        self.assertTrue(res.isNull())

    def testQDateValid(self):
        '''QDataStream <<>> QDate - valid'''
        self.stream << QDate(2012, 12, 21)

        res = QDate()

        self.read_stream >> res
        self.assertEqual(res, QDate(2012, 12, 21))
        self.assertTrue(res.isValid())
        self.assertFalse(res.isNull())

    def testQTimeNull(self):
        '''QDataStream <<>> QTime - null'''
        self.stream << QTime()

        res = QTime()

        self.read_stream >> res
        self.assertEqual(res, QTime())
        self.assertFalse(res.isValid())
        self.assertTrue(res.isNull())

    def testQTimeValid(self):
        '''QDataStream <<>> QTime - valid'''
        self.stream << QTime(12, 12, 21)

        res = QTime()

        self.read_stream >> res
        self.assertEqual(res, QTime(12, 12, 21))
        self.assertTrue(res.isValid())
        self.assertFalse(res.isNull())

    def testQDateTimeNull(self):
        '''QDataStream <<>> QDateTime - null'''

        self.stream << QDateTime()

        res = QDateTime()

        self.read_stream >> res
        self.assertEqual(res, QDateTime())
        self.assertFalse(res.isValid())
        self.assertTrue(res.isNull())

    def testQDateTimeValid(self):
        '''QDataStream <<>> QDateTime - valid'''
        time = QTime(23, 23, 23)
        date = QDate(2009, 1, 1)

        self.stream << QDateTime(date, time)

        res = QDateTime()

        self.read_stream >> res
        self.assertEqual(res, QDateTime(date, time))
        self.assertTrue(res.isValid())
        self.assertFalse(res.isNull())


class QDataStreamShiftBitArray(unittest.TestCase):
    def _check_bitarray(self, data_set):
        '''Check the >> operator for the given data set'''

        for data, expectedStatus, expectedString in data_set:
            stream = QDataStream(data, QIODevice.OpenModeFlag.ReadOnly)
            string = QBitArray()
            stream >> string

            self.assertEqual(stream.status(), expectedStatus)
            self.assertEqual(string.size(), expectedString.size())
            self.assertEqual(string, expectedString)

    def testOk(self):
        '''QDataStream with valid QBitArray'''
        test_set = [QBitArray()]
        for i in range(1, 7):
            test_set.append(create_bitarray(i * '1'))
        for s in ['0111111', '0000000', '1001110']:
            test_set.append(create_bitarray(s))

        data = []
        for expected in test_set:
            data.append((serialize_bitarray(expected), QDataStream.Status.Ok, expected))
        self._check_bitarray(data)

    def testPastEnd(self):
        '''QDataStream >> QBitArray reading past the end of the data'''
        serialized = serialize_bitarray(create_bitarray('1001110'))
        serialized.resize(serialized.size() - 2)
        self._check_bitarray([(serialized, QDataStream.Status.ReadPastEnd, QBitArray())])


class QDataStreamBuffer(unittest.TestCase):
    def testRawData(self):
        data = QDataStream()
        self.assertEqual(data.readRawData(4), None)

        ba = QByteArray()
        data = QDataStream(ba, QIODevice.OpenModeFlag.WriteOnly)
        data.writeRawData('AB\x00C')
        self.assertEqual(ba.data(), bytes('AB\x00C', "UTF-8"))

        data = QDataStream(ba)
        self.assertEqual(data.readRawData(4), bytes('AB\x00C', "UTF-8"))

    def testRawDataBytes(self):
        test_data = b'AB\0'
        data = QDataStream()
        ba = QByteArray()
        data = QDataStream(ba, QIODevice.OpenModeFlag.WriteOnly)
        data.writeRawData(test_data)
        self.assertEqual(ba.data(), test_data)
        data = QDataStream(ba)
        self.assertEqual(data.readRawData(3), test_data)

    def testBytes(self):
        dataOne = QDataStream()
        self.assertEqual(dataOne.readBytes(4), None)

        ba = QByteArray()
        data = QDataStream(ba, QIODevice.OpenModeFlag.WriteOnly)
        # writeBytes() writes a quint32 containing the length of the data,
        # followed by the data.
        data.writeBytes(bytes('AB\x00C', 'UTF-8'))
        self.assertEqual(ba.data(), bytes('\x00\x00\x00\x04AB\x00C', 'UTF-8'))

        data = QDataStream(ba)
        buffer = data.readBytes(4)
        self.assertEqual(buffer, bytes('AB\x00C', 'UTF-8'))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2009 Darryl Wallace, 2009 <wallacdj@gmail.com>
# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the itemviews/fetchmore/fetchmore example from Qt v6.x

Navigate to a directory with many entries by doubleclicking and scroll
down the list to see the model being populated on demand.
"""

import sys

from PySide6.QtCore import (QAbstractListModel, QDir,
                            QModelIndex, Qt, Signal, Slot)
from PySide6.QtWidgets import (QApplication, QFileIconProvider, QListView,
                               QPlainTextEdit, QSizePolicy, QVBoxLayout,
                               QWidget)


BATCH_SIZE = 100


class FileListModel(QAbstractListModel):

    number_populated = Signal(str, int, int, int)

    def __init__(self, parent=None):
        super().__init__(parent)

        self._path = ''
        self._file_count = 0
        self._file_list = []
        self._icon_provider = QFileIconProvider()

    def rowCount(self, parent=QModelIndex()):
        return self._file_count

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        if not index.isValid():
            return None

        row = index.row()
        if row >= len(self._file_list) or row < 0:
            return None

        if role == Qt.ItemDataRole.DisplayRole:
            return self._file_list[row].fileName()

        if role == Qt.ItemDataRole.BackgroundRole:
            batch = row // BATCH_SIZE
            palette = qApp.palette()  # noqa: F821
            return palette.base() if batch % 2 == 0 else palette.alternateBase()

        if role == Qt.ItemDataRole.DecorationRole:
            return self._icon_provider.icon(self._file_list[row])

        return None

    def canFetchMore(self, index):
        return self._file_count < len(self._file_list)

    def fetchMore(self, index):
        start = self._file_count
        total = len(self._file_list)
        remainder = total - start
        items_to_fetch = min(BATCH_SIZE, remainder)

        self.beginInsertRows(QModelIndex(), start, start + items_to_fetch)

        self._file_count += items_to_fetch

        self.endInsertRows()

        self.number_populated.emit(self._path, start, items_to_fetch, total)

    @Slot(str)
    def set_dir_path(self, path):
        self._path = path
        directory = QDir(path)

        self.beginResetModel()
        directory_filter = QDir.Filter.AllEntries | QDir.Filter.NoDot
        self._file_list = directory.entryInfoList(directory_filter, QDir.SortFlag.Name)
        self._file_count = 0
        self.endResetModel()

    def fileinfo_at(self, index):
        return self._file_list[index.row()]


class Window(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        self._model = FileListModel(self)
        self._model.set_dir_path(QDir.rootPath())

        self._view = QListView()
        self._view.setModel(self._model)

        self._log_viewer = QPlainTextEdit()
        self._log_viewer.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Preferred,
                                       QSizePolicy.Policy.Preferred))

        self._model.number_populated.connect(self.update_log)
        self._view.activated.connect(self.activated)

        layout = QVBoxLayout(self)
        layout.addWidget(self._view)
        layout.addWidget(self._log_viewer)

        self.setWindowTitle("Fetch More Example")

    @Slot(str, int, int, int)
    def update_log(self, path, start, number, total):
        native_path = QDir.toNativeSeparators(path)
        last = start + number - 1
        entry = f'{start}..{last}/{total} items from "{native_path}" added.'
        self._log_viewer.appendPlainText(entry)

    @Slot(QModelIndex)
    def activated(self, index):
        fileinfo = self._model.fileinfo_at(index)
        if fileinfo.isDir():
            self._log_viewer.clear()
            self._model.set_dir_path(fileinfo.absoluteFilePath())


if __name__ == '__main__':
    app = QApplication(sys.argv)

    window = Window()
    window.resize(400, 500)
    window.show()

    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys
from PySide6.QtCore import Qt, Slot
from PySide6.QtGui import QPainter
from PySide6.QtWidgets import (QApplication, QFormLayout, QHeaderView,
                               QHBoxLayout, QLineEdit, QMainWindow,
                               QPushButton, QTableWidget, QTableWidgetItem,
                               QVBoxLayout, QWidget)
from PySide6.QtCharts import QChartView, QPieSeries, QChart


class Widget(QWidget):
    def __init__(self):
        super().__init__()
        self.items = 0

        # Example data
        self._data = {"Water": 24.5, "Electricity": 55.1, "Rent": 850.0,
                      "Supermarket": 230.4, "Internet": 29.99, "Bars": 21.85,
                      "Public transportation": 60.0, "Coffee": 22.45, "Restaurants": 120}

        # Left
        self.table = QTableWidget()
        self.table.setColumnCount(2)
        self.table.setHorizontalHeaderLabels(["Description", "Price"])
        self.table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)

        # Chart
        self.chart_view = QChartView()
        self.chart_view.setRenderHint(QPainter.Antialiasing)

        # Right
        self.description = QLineEdit()
        self.description.setClearButtonEnabled(True)
        self.price = QLineEdit()
        self.price.setClearButtonEnabled(True)

        self.add = QPushButton("Add")
        self.clear = QPushButton("Clear")
        self.plot = QPushButton("Plot")

        # Disabling 'Add' button
        self.add.setEnabled(False)

        form_layout = QFormLayout()
        form_layout.addRow("Description", self.description)
        form_layout.addRow("Price", self.price)
        self.right = QVBoxLayout()
        self.right.addLayout(form_layout)
        self.right.addWidget(self.add)
        self.right.addWidget(self.plot)
        self.right.addWidget(self.chart_view)
        self.right.addWidget(self.clear)

        # QWidget Layout
        self.layout = QHBoxLayout(self)
        self.layout.addWidget(self.table)
        self.layout.addLayout(self.right)

        # Signals and Slots
        self.add.clicked.connect(self.add_element)
        self.plot.clicked.connect(self.plot_data)
        self.clear.clicked.connect(self.clear_table)
        self.description.textChanged.connect(self.check_disable)
        self.price.textChanged.connect(self.check_disable)

        # Fill example data
        self.fill_table()

    @Slot()
    def add_element(self):
        des = self.description.text()
        price = float(self.price.text())

        self.table.insertRow(self.items)
        description_item = QTableWidgetItem(des)
        price_item = QTableWidgetItem(f"{price:.2f}")
        price_item.setTextAlignment(Qt.AlignRight)

        self.table.setItem(self.items, 0, description_item)
        self.table.setItem(self.items, 1, price_item)

        self.description.clear()
        self.price.clear()

        self.items += 1

    @Slot()
    def check_disable(self, s):
        enabled = bool(self.description.text() and self.price.text())
        self.add.setEnabled(enabled)

    @Slot()
    def plot_data(self):
        # Get table information
        series = QPieSeries()
        for i in range(self.table.rowCount()):
            text = self.table.item(i, 0).text()
            number = float(self.table.item(i, 1).text())
            series.append(text, number)

        chart = QChart()
        chart.addSeries(series)
        chart.legend().setAlignment(Qt.AlignLeft)
        self.chart_view.setChart(chart)

    def fill_table(self, data=None):
        data = self._data if not data else data
        for desc, price in data.items():
            description_item = QTableWidgetItem(desc)
            price_item = QTableWidgetItem(f"{price:.2f}")
            price_item.setTextAlignment(Qt.AlignRight)
            self.table.insertRow(self.items)
            self.table.setItem(self.items, 0, description_item)
            self.table.setItem(self.items, 1, price_item)
            self.items += 1

    @Slot()
    def clear_table(self):
        self.table.setRowCount(0)
        self.items = 0


class MainWindow(QMainWindow):
    def __init__(self, widget):
        super().__init__()
        self.setWindowTitle("Tutorial")

        # Menu
        self.menu = self.menuBar()
        self.file_menu = self.menu.addMenu("File")

        # Exit QAction
        exit_action = self.file_menu.addAction("Exit", self.close)
        exit_action.setShortcut("Ctrl+Q")

        self.setCentralWidget(widget)


if __name__ == "__main__":
    # Qt Application
    app = QApplication(sys.argv)
    # QWidget
    widget = Widget()
    # QMainWindow using QWidget as central widget
    window = MainWindow(widget)
    window.resize(800, 600)
    window.show()

    # Execute application
    sys.exit(app.exec())



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import QFileSystemModel
from PySide6.QtQuick import QQuickTextDocument
from PySide6.QtQml import QmlElement, QmlSingleton
from PySide6.QtCore import (Qt, QDir, QAbstractListModel, Slot, QFile, QTextStream,
                            QMimeDatabase, QFileInfo, QStandardPaths, QModelIndex,
                            Signal, Property)

QML_IMPORT_NAME = "FileSystemModule"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
@QmlSingleton
class FileSystemModel(QFileSystemModel):

    rootIndexChanged = Signal()

    def getDefaultRootDir():
        return QStandardPaths.writableLocation(QStandardPaths.StandardLocation.HomeLocation)

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.mRootIndex = QModelIndex()
        self.mDb = QMimeDatabase()
        self.setFilter(QDir.Filter.AllEntries | QDir.Filter.Hidden | QDir.Filter.NoDotAndDotDot)
        self.setInitialDirectory()

    # check for the correct mime type and then read the file.
    # returns the text file's content or an error message on failure
    @Slot(str, result=str)
    def readFile(self, path):
        if path == "":
            return ""

        file = QFile(path)

        mime = self.mDb.mimeTypeForFile(QFileInfo(file))
        if ('text' in mime.comment().lower()
                or any('text' in s.lower() for s in mime.parentMimeTypes())):
            if file.open(QFile.OpenModeFlag.ReadOnly | QFile.OpenModeFlag.Text):
                stream = QTextStream(file).readAll()
                file.close()
                return stream
            else:
                return self.tr("Error opening the file!")
        return self.tr("File type not supported!")

    @Slot(QQuickTextDocument, int, result=int)
    def currentLineNumber(self, textDocument, cursorPosition):
        td = textDocument.textDocument()
        tb = td.findBlock(cursorPosition)
        return tb.blockNumber()

    def setInitialDirectory(self, path=getDefaultRootDir()):
        dir = QDir(path)
        if dir.makeAbsolute():
            self.setRootPath(dir.path())
        else:
            self.setRootPath(self.getDefaultRootDir())
        self.setRootIndex(self.index(dir.path()))

    # we only need one column in this example
    def columnCount(self, parent):
        return 1

    @Property(QModelIndex, notify=rootIndexChanged)
    def rootIndex(self):
        return self.mRootIndex

    def setRootIndex(self, index):
        if (index == self.mRootIndex):
            return
        self.mRootIndex = index
        self.rootIndexChanged.emit()


@QmlElement
class LineNumberModel(QAbstractListModel):

    lineCountChanged = Signal()

    def __init__(self, parent=None):
        self.mLineCount = 0
        super().__init__(parent=parent)

    @Property(int, notify=lineCountChanged)
    def lineCount(self):
        return self.mLineCount

    @lineCount.setter
    def lineCount(self, n):
        if n < 0:
            print("lineCount must be greater then zero")
            return
        if self.mLineCount == n:
            return

        if self.mLineCount < n:
            self.beginInsertRows(QModelIndex(), self.mLineCount, n - 1)
            self.mLineCount = n
            self.endInsertRows()
        else:
            self.beginRemoveRows(QModelIndex(), n, self.mLineCount - 1)
            self.mLineCount = n
            self.endRemoveRows()

    def rowCount(self, parent):
        return self.mLineCount

    def data(self, index, role):
        if not self.checkIndex(index) or role != Qt.ItemDataRole.DisplayRole:
            return
        return index.row()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ExceptionTest


class CppExceptionTest(unittest.TestCase):

    def testVoid(self):
        exceptionCount = 0
        et = ExceptionTest()

        et.voidThrowStdException(False)

        try:
            et.voidThrowStdException(True)
        except:  # noqa: E722
            exceptionCount += 1

        et.voidThrowInt(False)

        try:
            et.voidThrowInt(True)
        except:  # noqa: E722
            exceptionCount += 1

        self.assertEqual(exceptionCount, 2)

    def testReturnValue(self):
        exceptionCount = 0
        et = ExceptionTest()

        result = et.intThrowStdException(False)

        try:
            result = et.intThrowStdException(True)
        except:  # noqa: E722
            exceptionCount += 1

        result = et.intThrowInt(False)

        try:
            result = et.intThrowInt(True)  # noqa: F841
        except:  # noqa: E722
            exceptionCount += 1

        self.assertEqual(exceptionCount, 2)

        def testModifications(self):
            """PYSIDE-1995, test whether exceptions are propagated
               when return ownership modifications are generated."""
            exceptionCount = 0
            try:
                et = ExceptionTest.create(True)  # noqa: F841
            except:  # noqa: E722
                exceptionCount += 1
            self.assertEqual(exceptionCount, 1)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtCore import QDir, QUrl  # noqa: E402
from PySide6.QtGui import QGuiApplication  # noqa: E402
from PySide6.QtQml import qmlRegisterType  # noqa: E402


class TestQmlSupport(unittest.TestCase):

    def testIt(self):
        app = QGuiApplication([])  # noqa: F841

        file = os.fspath(Path(__file__).resolve().parent / 'ModuleType.qml')
        url = QUrl.fromLocalFile(QDir.fromNativeSeparators(file))
        result = qmlRegisterType(url, "CustomModule", 1, 0, "ModuleType")
        self.assertTrue(result != -1)


if __name__ == '__main__':
    unittest.main()
